(function($) {

    RAVEN.namespace("RAVEN.ExecuteTest");

    /**
     * This Model defines the status
     */
    RAVEN.ExecuteTest.StatusModel = Backbone.Model.extend({
        defaults: function() {
            return []
        }
    });
    
    function getReadonly() {
    	return $("#test-run-readonly").val() == 'true';
    }

    /**
     * This is the status view for the Execution page
     */
    RAVEN.ExecuteTest.StatusView = Backbone.View.extend({
        el: null,

        model: new RAVEN.ExecuteTest.StatusModel(),

        /**
         * The Test Run id
         */
        testRunId: null,

        /**
         * The Step id
         */
        stepId: null,

        /**
         * Trigger events on fetch success
         */
        triggerEvent: null,
        refreshSteps: null,

        events: {
            'click #testrun-status-action .transition': '_onTestRunUpdateStatus',
            'click .testStep .transition': '_onStepUpdateStatus',
            'step-status-update': '_stepStatusUpdated'
        },

        initialize: function(options) {
            var self = this;

            this.testRunId = $("#test-run-id").val();

            if (options) {
                _.extend(this, _.pick(options, "stepId"));
                _.extend(this, _.pick(options, "triggerEvent"));
                _.extend(this, _.pick(options, "refreshSteps"));
            }

            if(this.el.id === 'testrun-status-action') {
                this.model.url = contextPath + '/rest/raven/1.0/testrun/' + this.testRunId + "/status";
            } else {
                this.model.url = contextPath + '/rest/raven/1.0/testrun/' +  + this.testRunId + '/steps/' + this.stepId + '/status';
            }

            self.$el.block();
            this.model.fetch({
                success: function() {
                    self.render();
                    if(self.triggerEvent) {
                        $(self.el).trigger(self.triggerEvent, [{
                            data: self.model.attributes,
                            refreshSteps: self.refreshSteps
                        }]);
                    }
                    self.$el.unblock();
                },
                error: function(model, response) {
                    self.$el.unblock();
                }
            });
        },

        render: function() {
            this.$el.empty();

            var content = RAVEN.ExecuteTest.status({
                status: this.model.get('obj').status,
                transitions: this.model.get('obj').transitions,
                readonly: getReadonly()
            });

            this.$el.html(content);
            
            this.applyTipsy();

            this.$el.show();

            return this;
        },

        refresh: function() {
            this.initialize();
        },

        _onTestRunUpdateStatus: function(event) {
            var self = this;

            // New Status Id
            var data = $(event.target).attr("value");

            self.$el.block();
            $.ajax({
                type: "PUT",
                url:  contextPath + '/rest/raven/1.0/testrun/' + self.testRunId + "/status",
                processData: true,
                async: true,
                data: data,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    self.message(AJS.I18n.getText('execute-test-action.status.check.error'), errorThrown, 'fatal');
                },
                success: function(data, textStatus, jqXHR) {
                    self.initialize({
                        "triggerEvent" : "testrun-status-update",
                        'refreshSteps': true
                    });
                },
                complete: function() {
                    self.$el.unblock();
                }
            });
        },

        _onStepUpdateStatus: function(event) {
            var self = this;

            // New Status Id
            var data = $(event.target).data('status');

            self.$el.block();
            $.ajax({
                type: "PUT",
                url: contextPath + '/rest/raven/1.0/testrun/' + self.testRunId + '/steps/' + self.stepId + '/status',
                processData: true,
                data: data,
                async: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    self.message(AJS.I18n.getText('execute-test-action.status.check.error'), errorThrown);
                },
                success: function(data, textStatus, jqXHR) {
                    self.refresh();
                    //Every time we refresh a step status, we need to refresh the Test Run status
                    $('#testrun-status-action').trigger('step-status-update');
                    $('#testrun-status-action').trigger('step-update');
                    // colors the step index and the step module
                    // this is needed because we can't access the main model of the step. We only have access to the collection
                    $('.testStep[data-step-id=' + self.stepId + '] .testStepIndex').css('background-color', 'rgb(' + data.obj.status.colorRGB + ')');
                    $('.testStep[data-step-id=' + self.stepId + '] #steps-module-' + self.stepId).css('background-color', 'rgba(' + data.obj.status.colorRGB + ',0.08)');
                },
                complete: function() {
                    self.$el.unblock();
                }
            });
        },

        _stepStatusUpdated: function() {
            this.initialize({
                "triggerEvent" : "testrun-status-update",
                'refreshSteps': false
            });
        },

        message: function (title, message, type) {
            type = (type === undefined || type == "") ? "error" : type;

            if (!message && title) {
                message = title;
                title = null;
            }

            if (type === "fatal") {
                this._clearMessage();
                AJS.messages.error('#raven-execute-test-message-bar-info', {
                    title: title,
                    body: message,
                    closeable: false
                });
                this.$el.hide();
            }

            if (type === "error") {
                this._clearMessage();
                AJS.messages.error('#raven-execute-test-message-bar-info', {
                    title: title,
                    body: message
                });
            }

            if (type === "success") {
                AJS.messages.success('#raven-execute-test-message-bar-info', {
                    title: title,
                    body: message
                });
            }

            if (type === "info") {
                AJS.messages.info('#raven-execute-test-message-bar-info', {
                    title: title,
                    body: message
                });
            }
        },

        _clearMessage: function() {
            $('#raven-execute-test-message-bar-info').empty();
        },
        
        applyTipsy: function() {
        	$(".tipsy").remove();
        	$(".status-tipsy").tipsy();
        }
    });
})(AJS.$);