(function ($) {

    var gherkinTheme = require("ace/theme/textmate");

    window.RAVEN = window.RAVEN || {};

    RAVEN.InlineEdit = RAVEN.InlineEdit || {};

    var SELECT_FIELD_EMPTY_LABEL = "None";
    var SELECT_FIELD_EMPTY_ASSIGNEE = "-";

    RAVEN.InlineEdit = Backbone.Model.extend({
        initialize: function() {
            var self = this;
            
            $(document).off('inline-edit');

            $(document).on("click.inline-edit", 'body', function(e) {
                if ($(e.target).closest('.raven-editable-field.active').length == 0) {
                    var $activeField = $('.raven-editable-field.active');
                    if ($activeField.length > 0 && !$activeField.hasClass('working')) {
                        var field = self._getFieldFromElement($activeField);

                        // do not save field if click started on ace editor
                        if (field.aceEditor && field.mouseDown === true) {
                            field.mouseDown = null;
                            return true;
                        }
                        
                        field.save();
                    }
                }
            });

            $(document).on("mousedown.inline-edit", '.raven-editable-field.active', function(e) {
                var $activeField = $(this);

                if ($activeField.length > 0) {
                    var field = self._getFieldFromElement($activeField);
                    // check if mousedown was on ace editor
                    if (field.aceEditor) {
                        field.mouseDown = true;
                    }
                }
            });

            $(document).on("mouseup.inline-edit", '.raven-editable-field.active', function(e) {
                var $activeField = $('.raven-editable-field.active');

                if ($activeField.length > 0) {
                    var field = self._getFieldFromElement($activeField);
                    // check if mousedown was on ace editor
                    if (field.aceEditor) {
                        field.mouseDown = null;
                    }
                }
            });
            
            $(document).on('click.inline-edit', '.raven-editable-field', function(e) {
                // if there is an active editable field and it's not this, save the active field
                var $activeField = $('.raven-editable-field.active');
                var $this = $(this);

                if ($activeField.length > 0 && $activeField[0] != $this[0]) {
                    self._getFieldFromElement($activeField).save();
                }
                if ($this.hasClass('inactive') && $this.hasClass('enable')) {

                    var field = self._getFieldFromElement($this);

                    // do not enter in edit mode when click is a selection
                    var selectedText = null;
                    if (field.aceEditor) {
                        selectedText = field.aceEditor.getSelectedText()
                    } else {
                        selectedText = getSelection().toString();
                    }
                    
                    if(!selectedText) {
                        field.edit();
                    }
                }
            });
            
            $(document).on('click.inline-edit', '.raven-editable-field.enable.active .submit', function(e) {
                var field = self._getFieldFromElement($(this).closest('.raven-editable-field'));
                if (field) {
                    field.save();
                }
                // prevent default posting of form
                e.preventDefault();
                e.stopPropagation();
            });
            
            $(document).on('click.inline-edit', '.raven-editable-field.enable.active .cancel', function(e) {
                // prevent default posting of form
                e.preventDefault();
                e.stopPropagation();

                var field = self._getFieldFromElement($(this).closest('.raven-editable-field'));
                if (field) {
                    field.cancel();
                }

                return false;
            });
            
            $(document).on('click.inline-edit', '.aui-field-wikiedit.path-input .submit', function(e) {
                var field = self._getFieldFromElement($(this).closest(".aui-field-wikiedit").parent().find("#test-generic-field"));
                if (field) {
                    field.save();
                }
                // prevent default posting of form
                e.preventDefault();
                e.stopPropagation();
            });
            
            $(document).on('click.inline-edit', '.aui-field-wikiedit.path-input .cancel', function(e) {
                // prevent default posting of form
                e.preventDefault();
                e.stopPropagation();

                var field = self._getFieldFromElement($(this).closest(".aui-field-wikiedit").parent().find("#test-generic-field"));
                if (field) {
                    field.cancel();
                }

                return false;
            });
        },

        _getFieldFromElement: function($el) {
            if ($el.length > 0) {
                return $el[0].inlineMeta;
            } else {
                return null;
            }
        }
    });
    
    // Default base field
    RAVEN.InlineEdit.Field = Backbone.Model.extend({
        initialize: function(elem, callbacks) {
            this.el = $(elem)[0];
            this.$field = $(elem);
            this.el.inlineMeta = this;
            this.path = this.$field.data('field-url');
            
            this.initValue();

            if (callbacks) {
                this.fnValidate = callbacks.validate;
                this.fnBeforeCallback = callbacks.before;
                this.fnSuccessCallback = callbacks.success;
                this.fnOnError = callbacks.error;
            }

            // render field
            this.render();

            // show only after render
            this.$field.css('visibility', 'visible');
        },

        initValue: function() {
            this.value = '' + this.$field.data('field-value'); // make sure it stores a string
        },

        _emptyPlaceHolder: function() {
            var enabled = this.$field.hasClass('enable');
            if (enabled) {
                return '<em>' + this.$field.data('field-place-holder') + '</em>';
            } else {
                return AJS.I18n.getText('common.label.none');
            }
        },

        // render initial field content
        render: function() {
            this.$field.addClass('raven-editable-field inactive');
            // HACK for IE8
            if (navigator.userAgent.match(/MSIE 8/) !== null) {
                var self = this;
                this.$field.hover(function() {
                    self.$field.addClass("msie-8");
                },function() {
                    self.$field.removeClass("msie-8");
                });
            }
        },

        save: function() {
            var self = this;
            
            if (!this.$field.hasClass('working')) {
                // disable controls
                this.toogleControls(false);

                if (this.hasChanges()) {
                    if (this.fnValidate) {
                        this.fnValidate(function() { self._saveCallback(); }, function() { self.cancel(); })
                    } else {
                        this._saveCallback();
                    }
                } else {
                    this.close();
                }
            }
        },
        
        _saveCallback: function() {
            var self = this;
            var value = this.getSubmitValue();
            this._putField(
                value,
                function() {
                    var makeRequest = true;
                    if (self.fnBeforeCallback) {
                        makeRequest = self.fnBeforeCallback();
                    }
                    
                    if (makeRequest === true) {
                        // disable controls
                        self.toogleControls(false);
                    } else {
                        self.cancel();
                    }
                    return makeRequest;
                },
                function() {
                    // enable fields
                    self.toogleControls(true);
                }, 
                function(data, textStatus, jqXHR) {
                    self._handleSaveSuccess(data, textStatus, jqXHR);
                },
                function() {
                    self.cancel();
                });
        },

        _handleSaveSuccess: function(data, textStatus, jqXHR) {
            var self = this;
            // save new curent value
            self.value = self.getEditingValue();
            self.$field.data('field-value', self.value);

            if (self.fnSuccessCallback) {
                self.fnSuccessCallback(data);
            }
            self.close();
        },

        _resetField: function() {
            var self = this;
            
            this._putField(
                null,
                null,
                function() {
                    // enable fields
                    self.toogleControls(true);
                }, 
                function(data, textStatus, jqXHR) {
                    // save new current value
                    self.$field.data('field-value', self.value);

                    if (self.fnSuccessCallback) {
                        self.fnSuccessCallback(data);
                    }
                    self.close();
                }, 
                function() {
                    self.cancel();
                });
        },
        
        _putField: function(value, before, complete, success, fail) {
            var self = this;
            var data = null;
            
            // check if this is a JIRA custom field
            var fieldId = this.$field.data('field-id');
            if (fieldId) {
                var fieldObj = {};
                fieldObj[fieldId] = value;
                var value = {
                    "fields" : fieldObj
                };
                data = JSON.stringify(value);
            } else {
                data = value;
            }
            
            $.ajax({
                type: "PUT",
                url: this.path,
                processData: false,
                async: true,
                contentType: "application/json",
                data: data,
                beforeSend: before,
                success: success,
                error: function(qXHR, textStatus, errorThrown) {
                    fail();
                    
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    
                    if (self.fnOnError) {
                        self.fnOnError(this, errorThrown);
                    }
                },
                complete : complete
            });
        },
        
        getValue: function() {
            return this.value;
        },

        setValue: function(value) {
            this.value = value;

            this.renderValue(value);
        },

        // implemented by clients
        renderValue: function(value) {
        },

        // implemented by clients
        getEditingValue: function() {
            return null;
        },

        // implemented by clients
        getSubmitValue: function() {
            return this.getEditingValue();
        },

        edit: function() {
            this.$field.removeClass('inactive').addClass('active');
            this.$field.children('.field-view').hide();
            
            this.$field.children('.field-edit').show();
        },

        view: function() {
            this.renderValue(this.value);
            
            // enable controls
            this.toogleControls(true);
            
            this.$field.children('.field-edit').hide();
            this.$field.removeClass('active working').addClass('inactive');
            this.$field.children('.field-view').show();
        },

        reset: function() {
            this.value = null;
            
            this._resetField();
        },

        close: function() {
            this.view();
        },

        cancel: function() {
            this.view();
        },
        
        // implemented by clients
        toogleControls: function() {
        },

        hasChanges: function() {
            var previousValue = this.getValue();
            var value = this.getEditingValue();
            
            return value !== previousValue;
        }
    });

    /*
     * Text Field
     */
    RAVEN.InlineEdit.TextField = RAVEN.InlineEdit.Field.extend({
        initialize: function(elem, callbacks) {
            this.$input = $(elem).children('input, textarea');

            // call parent constructor
            RAVEN.InlineEdit.TextField.__super__.initialize.apply(this, [elem, callbacks]);
        },

        render: function() {
            RAVEN.InlineEdit.TextField.__super__.render.apply(this);

            var $content = this.$field.children();

            this.$field.append(RAVEN.InlineEdit.FieldRenderer.renderTextField({'value': this.value, 'emptyPlaceHolder': this._emptyPlaceHolder()} ));

            $content.detach().appendTo(this.$field.find('.field-control'));
        },

        getEditingValue: function() {
            var value = this.$input.val();
            return value;
        },

        edit: function() {
            RAVEN.InlineEdit.TextField.__super__.edit.apply(this);
            
            var value = this.getValue();        
            this.$input.val(value);
            this.$input.focus();
        },

        renderValue: function(value) {
            var $content = this.$field.find('.field-view .field-content')
            if (!value || $.trim(value) == '') {
                $content.empty().append('<span class="field-place-holder">' + this._emptyPlaceHolder() + '</span>');
            } else {
                $content.text(value);
            }
        },

        cancel: function() {
            // set the old value
            this.$input.val(this.getValue());

            RAVEN.InlineEdit.TextField.__super__.cancel.apply(this);
        },
    
        toogleControls: function(enable) {
            this.$input.prop("disabled", !enable);
            this.$field.find('.save-options button').prop("disabled", !enable);
            
            if (!enable) {
                this.$field.addClass('working');
            } else {
                this.$field.removeClass('working');
            }
        }
    });

    /*
     * Wiki Text Field
     */
    RAVEN.InlineEdit.WikiTextField = RAVEN.InlineEdit.TextField.extend({
        initialize: function(elem, callbacks) {
            // call parent constructor
            RAVEN.InlineEdit.WikiTextField.__super__.initialize.apply(this, [elem, callbacks]);
        },

        initValue: function() {
        	this.value = {
        			raw: this.$field.data('field-value'),
        			rendered: this.$field.data('field-rendered')
            }
        },

        render: function() {
            RAVEN.InlineEdit.TextField.__super__.render.apply(this);

            var $content = this.$field.children();
/*
            var decode = function htmlDecode(value) { 
                return (typeof value === 'undefined') ? '' : $('<div/>').html(value).text(); 
            };
            */

            this.$field.append(RAVEN.InlineEdit.FieldRenderer.renderWikiTextField({'value': this.value.rendered, 'emptyPlaceHolder': this._emptyPlaceHolder()} ));

            $content.detach().appendTo(this.$field.find('.field-control'));
        },

        setValue: function(newValue) {
            if (typeof newValue == 'string' || newValue instanceof String) {
                var self = this;
                
                if (newValue) {
                    self.toogleControls(false);
                    self._renderWikiContent([newValue], function(renderedValues) {
                        if (renderedValues && renderedValues.length > 0) {
                            self.value = {
                                raw: newValue,
                                rendered: renderedValues[0]
                            };

                            self.renderValue(self.value);
                        }
                    },
                    null,
                    function() {
                        self.toogleControls(true);
                    });
                } else {
                    self.value = {
                        raw: null,
                        rendered: null
                    };
                    self.renderValue(self.value);
                }
            } else if (typeof newValue == 'object') {
                this.value = newValue;

                this.renderValue(this.value);
            }
        },

        _renderWikiContent: function(rawValues, onSuccess, onFail, onComplete) {
            $.ajax({
                type: "POST",
                url: AJS.contextPath() + '/rest/raven/1.0/text/wiki', 
                data: JSON.stringify(rawValues), 
                contentType: "application/json",
                success: onSuccess,
                fail: onFail,
                complete: onComplete
            });
        },

        edit: function() {
            RAVEN.InlineEdit.TextField.__super__.edit.apply(this);
            
            var value = this.getValue();
            this.$input.val(value.raw);
            this.$input.focus();
        },

        renderValue: function(value) {
            var $content = this.$field.find('.field-view .field-content');
            if (!value || $.trim(value.raw) == '') {
                $content.empty().append('<span class="field-place-holder">' + this._emptyPlaceHolder() + '</span>');
            } else {
                $content.html(value.rendered);
            }
        },

        hasChanges: function() {
            var previousValue = this.getValue();
            var value = this.getEditingValue();
            
            return value !== previousValue.raw;
        },

        cancel: function() {
            // set the old value
            //this.$input.val(this.getValue().raw);

            RAVEN.InlineEdit.TextField.__super__.cancel.apply(this);
        },

        _handleSaveSuccess: function(data, textStatus, jqXHR) {
            // save new current value
            if (data && data.raw != undefined && data.rendered != undefined) {
                this.value = data;
                this.$field.data('field-value', this.value.raw);
                this.$field.data('field-rendered', this.value.rendered);

                if (this.fnSuccessCallback) {
                    this.fnSuccessCallback(data);
                }
                this.close();
            } else {
                throw 'Unexpected successfull result returned from server!';
            }
        }
    });
    
    /*
     * Jira Wiki Text Field
     */
    RAVEN.InlineEdit.JiraWikiTextField = RAVEN.InlineEdit.TextField.extend({
        initialize: function(elem, wikiEdit, callbacks) {
        	this.$wikiEdit = $(wikiEdit);
            // call parent constructor
            RAVEN.InlineEdit.WikiTextField.__super__.initialize.apply(this, [elem, callbacks]);
        },

        initValue: function() {
        	this.value = {
        		raw: this.$field.data('field-value'),
        		rendered : ""
            }
        		
        	this.setValue(this.value.raw);
        },

        render: function() {
            RAVEN.InlineEdit.TextField.__super__.render.apply(this);

            var $content = this.$field.children();
/*
            var decode = function htmlDecode(value) { 
                return (typeof value === 'undefined') ? '' : $('<div/>').html(value).text(); 
            };
            */

            this.$field.append(RAVEN.InlineEdit.FieldRenderer.renderWikiTextField({'value': this.value.rendered, 'emptyPlaceHolder': this._emptyPlaceHolder()} ));

            $content.detach().appendTo(this.$field.find('.field-control'));
        },
        
        getSubmitValue: function() {
        	return this.$wikiEdit.find("textarea").val();
        },

        setValue: function(newValue) {
            if (typeof newValue == 'string' || newValue instanceof String) {
                var self = this;
                
                if (newValue) {
                    self.toogleControls(false);
                    self._renderWikiContent([newValue], function(renderedValues) {
                        if (renderedValues && renderedValues.length > 0) {
                            self.value = {
                                raw: newValue,
                                rendered: renderedValues[0]
                            };

                            self.renderValue(self.value);
                        }
                    },
                    null,
                    function() {
                        self.toogleControls(true);
                    });
                } else {
                    self.value = {
                        raw: null,
                        rendered: null
                    };
                    self.renderValue(self.value);
                }
            } else if (typeof newValue == 'object') {
                this.value = newValue;

                this.renderValue(this.value);
            }
        },

        _renderWikiContent: function(rawValues, onSuccess, onFail, onComplete) {
            $.ajax({
                type: "POST",
                url: AJS.contextPath() + '/rest/raven/1.0/text/wiki', 
                data: JSON.stringify(rawValues), 
                contentType: "application/json",
                success: onSuccess,
                fail: onFail,
                complete: onComplete
            });
        },

        edit: function() {
            this.$wikiEdit.find("textarea").val(this.getValue().raw);
            this.$wikiEdit.show();
            this.$field.hide();
            
        },

        renderValue: function(value) {
            var $content = this.$field.find('.field-view .field-content');
            if (!value || $.trim(value.raw) == '') {
                $content.empty().append('<span class="field-place-holder">' + this._emptyPlaceHolder() + '</span>');
            } else {
                $content.html(value.rendered);
            }
        },

        hasChanges: function() {
            var previousValue = this.getValue();
            var value = this.getEditingValue();
            
            return value !== previousValue.raw;
        },

        cancel: function() {
            // set the old value
            //this.$input.val(this.getValue().raw);

        	this.$wikiEdit.hide();
            this.$field.show();
        },

        _handleSaveSuccess: function(data, textStatus, jqXHR) {
            // save new current value
            if (this.$wikiEdit) {
                this.value = this.$wikiEdit.find("textarea").val();
                this.$field.data('field-value', this.value);

                this.setValue(this.value);
                
            	this.$wikiEdit.hide();
                this.$field.show();
                
                if (this.fnSuccessCallback) {
                    this.fnSuccessCallback();
                }
                this.close();
            } else {
                throw 'Unexpected successfull result returned from server!';
            }
        }
    });

    /*
     * Jira Wiki Text Field
     */
    RAVEN.InlineEdit.JiraRichTextField = RAVEN.InlineEdit.TextField.extend({
        initialize: function(elem, wikiEdit, callbacks) {
            this.$wikiEdit = $(wikiEdit);
            // call parent constructor
            RAVEN.InlineEdit.WikiTextField.__super__.initialize.apply(this, [elem, callbacks]);
        },

        initValue: function() {
            this.value = {
                raw: this.$field.data('field-value'),
                rendered : ""
            }

            this.setValue(this.value.raw);
        },

        render: function() {
            RAVEN.InlineEdit.TextField.__super__.render.apply(this);

            var $content = this.$field.children();
            /*
             var decode = function htmlDecode(value) {
             return (typeof value === 'undefined') ? '' : $('<div/>').html(value).text();
             };
             */

            this.$field.append(RAVEN.InlineEdit.FieldRenderer.renderWikiTextField({'value': this.value.rendered, 'emptyPlaceHolder': this._emptyPlaceHolder()} ));

            $content.detach().appendTo(this.$field.find('.field-control'));
        },

        getSubmitValue: function() {
            return this.$wikiEdit.find("textarea").val();
        },

        setValue: function(newValue) {
            if (typeof newValue == 'string' || newValue instanceof String) {
                var self = this;

                if (newValue) {
                    self.toogleControls(false);
                    self._renderWikiContent([newValue], function(renderedValues) {
                            if (renderedValues && renderedValues.length > 0) {
                                self.value = {
                                    raw: newValue,
                                    rendered: renderedValues[0]
                                };

                                self.renderValue(self.value);
                            }
                        },
                        null,
                        function() {
                            self.toogleControls(true);
                        });
                } else {
                    self.value = {
                        raw: null,
                        rendered: null
                    };
                    self.renderValue(self.value);
                }
            } else if (typeof newValue == 'object') {
                this.value = newValue;

                this.renderValue(this.value);
            }
        },

        _renderWikiContent: function(rawValues, onSuccess, onFail, onComplete) {
            $.ajax({
                type: "POST",
                url: AJS.contextPath() + '/rest/raven/1.0/text/wiki',
                data: JSON.stringify(rawValues),
                contentType: "application/json",
                success: onSuccess,
                fail: onFail,
                complete: onComplete
            });
        },

        edit: function() {
            var self = this;
            var rawData = this.getValue().raw;
            this.$wikiEdit.find("textarea").val(rawData);


            var renderData = {
                forWysiwyg: true,
                issueKey: JIRA.Issue.getIssueKey(),
                rendererType: "atlassian-wiki-renderer",
                unrenderedMarkup: rawData
            };

                this.$field.block();
                $.ajax({
                    type: "POST",
                    url: AJS.contextPath() + '/rest/api/1.0/render',
                    contentType: "application/json",
                    data: JSON.stringify(renderData),
                    success: function(data) {

                        if(self.$wikiEdit.find(".rte-container").length == 0 ) {
                            self.$wikiEdit.find(".wiki-edit-content").append('<rich-editor contenteditable="true" data-issue-key="'+JIRA.Issue.getIssueKey()+'" data-content-present="true">'+ data +'</rich-editor>');
                        } else {
                            self.$wikiEdit.find("rich-editor").html(data);
                        }

                        self.$wikiEdit.find("textarea")[0].style.setProperty("max-width", "calc(100% - 20px)","important");
                        self.$wikiEdit.show();
                        self.$field.hide();
                    },
                    error: function(qXHR, textStatus, errorThrown) {
                        if (errorThrown && errorThrown.trim() != '') {
                            errorThrown = errorThrown + ': ' + qXHR.responseText;
                        } else {
                            errorThrown = qXHR.responseText;
                        }

                        if (self.fnOnError) {
                            self.fnOnError(this, errorThrown);
                        }
                    },
                    complete: function () {
                        self.$field.unblock();
                    }
                });
        },

        renderValue: function(value) {
            var $content = this.$field.find('.field-view .field-content');
            if (!value || $.trim(value.raw) == '') {
                $content.empty().append('<span class="field-place-holder">' + this._emptyPlaceHolder() + '</span>');
            } else {
                $content.html(value.rendered);
            }
        },

        hasChanges: function() {
            var previousValue = this.getValue();
            var value = this.getEditingValue();

            return value !== previousValue.raw;
        },

        cancel: function() {
            // set the old value
            //this.$input.val(this.getValue().raw);

            this.$wikiEdit.hide();
            this.$field.show();
        },

        _handleSaveSuccess: function(data, textStatus, jqXHR) {
            // save new current value
            if (this.$wikiEdit) {
                this.value = this.$wikiEdit.find("textarea").val();
                this.$field.data('field-value', this.value);

                this.setValue(this.value);

                this.$wikiEdit.hide();
                this.$field.show();

                if (this.fnSuccessCallback) {
                    this.fnSuccessCallback();
                }
                this.close();
            } else {
                throw 'Unexpected successfull result returned from server!';
            }
        }
    });

    /*
     * Select Field
     */
    RAVEN.InlineEdit.SelectField = RAVEN.InlineEdit.Field.extend({
        initialize: function(elem, callbacks) {
            // call parent constructor
            RAVEN.InlineEdit.SelectField.__super__.initialize.apply(this, [elem, callbacks]);
        },

        render: function() {
            RAVEN.InlineEdit.SelectField.__super__.render.apply(this);

            var $selectInput = this.$field.children('select');
            var label = $selectInput.find('option[value = "' + this.value + '"]').attr('label');

            if (!label) {
                label = SELECT_FIELD_EMPTY_LABEL;
            }

            this.$field.append(RAVEN.InlineEdit.FieldRenderer.renderSelectField({'label': label}));
            $selectInput.appendTo(this.$field.find('.field-control'));
        },

        setValue: function() {
            this.value = value;

            this.$field.find('.field-edit select').val(value);

            this.renderValue(value);
        },

        getEditingValue: function() {
            var value = this.$field.find('.field-edit select').val();
            if (value instanceof Array) {
                if (value.length > 0) {
                    return value[0];
                } else {
                    return null;
                }
            }
            return value;
        },
        
        edit: function() {
            RAVEN.InlineEdit.SelectField.__super__.edit.apply(this);
            // aahhhrr ... doesn't work
            //this.$field.children('select').click();

            this.$field.children('select').focus();
        },
        
        cancel: function() {
            // select the old value
            this.$field.find('.field-edit select').val(this.getValue());

            RAVEN.InlineEdit.SelectField.__super__.cancel.apply(this);
        },
    
        toogleControls: function(enable) {
            this.$field.find('select').prop("disabled", !enable);
            if (enable === true) {
                this.$field.find('.status-save-options a').show();
                this.$field.removeClass('working');
            } else {
                this.$field.find('.status-save-options a').hide();
                this.$field.addClass('working');
            }
        }
    });

    /*
     * Select Picker Field
     */

    RAVEN.InlineEdit.SelectPickerField = RAVEN.InlineEdit.SelectField.extend({
        initialize: function(elem, callbacks) {
            // call parent constructor
            RAVEN.InlineEdit.SelectPickerField.__super__.initialize.apply(this, [elem, callbacks]);

            // create JIRA select picker
            var $selectInput = this.$field.find('.field-control select');
            this.picker = this._createSelectPicker($selectInput);

            // do not propagate click event on suggestions
            var $select = this.$field.find('.field-edit select');
            $(document).on("click.inline-edit", '#' + $select.attr('id') + '-suggestions', function(e) {
                e.stopPropagation();
            });
        },

        setValue: function(value) {
            this.value = value;

            this.$field.find('.field-edit select').val(value);
            var $selectedOption = this.$field.find('.field-edit select option:selected');
            if ($selectedOption.length > 0) {
                var selected = new AJS.ItemDescriptor({
                    value: $selectedOption.val(),
                    label: $selectedOption.attr('label'),
                    selected: 'selected'
                });
                this.picker.setSelection(selected);
            }

            this.renderValue(value);
        },

        revertPicker: function() {
            // remove old picker
            var $el = this.picker.$container;
            $el.remove();

            var $selectInput = this.$field.find('.field-control select');
            this.picker = this._createSelectPicker($selectInput);
        },

        _createSelectPicker: function($selectField) {
            return new AJS.SingleSelect({
                element: $selectField,
                itemAttrDisplayed: "label",
                showDropdownButton: true,
                removeOnUnSelect: false,
                submitInputVal: true,
                revertOnInvalid: true
            });
        },

        getSubmitValue: function() {
            var optionbyId = this.$field.data('field-option-id');
            var value = this.getEditingValue();
            if (optionbyId && value) {
                return { 'id' : value };
            } else {
                return { 'value' : value };
            }
        },

        renderValue: function(value) {
            var label = this.$field.find('.field-edit select option:selected').attr('label');
            if (!label) {
                label = SELECT_FIELD_EMPTY_LABEL;
            }

            this.$field.find('.field-view .field-content').empty().append('<strong>' + label  + '</strong>');
        },

        edit: function() {
            RAVEN.InlineEdit.SelectPickerField.__super__.edit.apply(this);
            this.$field.find('input').focus();
        },

        cancel: function() {
            RAVEN.InlineEdit.SelectPickerField.__super__.cancel.apply(this);

            // select the old value
            var $selectedOption = this.$field.find('.field-edit select option:selected');
            if ($selectedOption.length > 0) {
                var selected = new AJS.ItemDescriptor({
                    value: $selectedOption.val(),
                    label: $selectedOption.attr('label'),
                    selected: 'selected'
                });
                this.picker.setSelection(selected);
            }
        }
    });

    /*
     * Ace Editor Field
     */
    RAVEN.InlineEdit.AceEditorField = RAVEN.InlineEdit.Field.extend({
        initialize: function(elem, callbacks) {
            // call parent constructor
            RAVEN.InlineEdit.AceEditorField.__super__.initialize.apply(this, [elem, callbacks]);

            this.editorElemId = this.$field.data('field-editor-id');
            var self = this;
            require(['ace/ace'], function(ace) {
                if (document.getElementById(self.editorElemId)) {
                    self.aceEditor = self._createAceEditor(ace, elem);
                }
            });
        },

        render: function() {
            RAVEN.InlineEdit.AceEditorField.__super__.render.apply(this);

            var editorElemId = this.$field.data('field-editor-id');

            this.$field.append(RAVEN.InlineEdit.FieldRenderer.renderAceEditorField({'editorId': editorElemId, 'value': this.value, 'emptyPlaceHolder': this._emptyPlaceHolder()}));
        },

        _createAceEditor: function(ace, elem) {
            var aceEditor = ace.edit(this.editorElemId);
            aceEditor.setTheme(gherkinTheme);
            aceEditor.setFontSize(14);
            aceEditor.getSession().setMode("ace/mode/gherkin");
            aceEditor.getSession().setUseWrapMode(false);
            aceEditor.renderer.setShowGutter(true);
            aceEditor.renderer.setShowPrintMargin(false);
            aceEditor.setReadOnly(true);
            aceEditor.setOptions({
                maxLines: 500 // Infinity - it is advisable to maintain a limit for very large documents
            });

            aceEditor.getSession().setValue(this.value);

            return aceEditor;
        },

        setValue: function(value) {
            this.value = value;

            this.aceEditor.getSession().setValue(value);

            this.renderValue(value);
        },

        getEditingValue: function() {
            return this.aceEditor.getSession().getValue();
        },
        
        edit: function() {
            RAVEN.InlineEdit.AceEditorField.__super__.edit.apply(this);
            
            // move editor from view to edit section
            var $editor = this.$field.find('#' + this.editorElemId);
            $editor.detach().appendTo('.field-edit form fieldset .ace-editor-wrapper');
            $editor.removeClass('ace-mode-viewer').addClass('ace-mode-editor');
            
            var value = this.getValue();        
            this.aceEditor.getSession().setValue(value);
            this.aceEditor.setReadOnly(false);
            this.aceEditor.renderer.setShowGutter(true);
            this.aceEditor.renderer.setShowPrintMargin(true);
            this.aceEditor.setOptions({
                maxLines: 30
            });
            
            this.aceEditor.focus();
            // focus is not working !!!
            //this.$field.find('.ace_editor').focus();

        },

        view: function() {
            // move editor from view to edit section
            var $editor = this.$field.find('#' + this.editorElemId);
            $editor.detach().appendTo('.field-view .ace-editor-wrapper');
            
            this.aceEditor.setReadOnly(true);
            this.aceEditor.renderer.setShowGutter(true);
            this.aceEditor.renderer.setShowPrintMargin(false);
            this.aceEditor.setOptions({
                maxLines: 500 // Infinity - it is advisable to maintain a limit for very large documents
            });
            $editor.removeClass('ace-mode-editor').addClass('ace-mode-viewer');
            
            RAVEN.InlineEdit.AceEditorField.__super__.view.apply(this);

        },
        
        renderValue: function(value) {
            // update value
            this.aceEditor.getSession().setValue(value);
            this.aceEditor.resize();

            // remove empty place holder, if exists
            this.$field.find('.field-view .field-content .field-place-holder').remove();
            if (!value || $.trim(value) == '') {
                this.$field.find('.field-view .ace-editor-wrapper').hide();
                this.$field.find('.field-view .field-content').append('<span class="field-place-holder">' + this._emptyPlaceHolder() + '</span>');
            } else {
                this.$field.find('.field-view .ace-editor-wrapper').show();
            }
        },

        cancel: function() {
            // set current value
            this.aceEditor.getSession().resetCaches();
            this.aceEditor.getSession().setValue(this.value);

            RAVEN.InlineEdit.AceEditorField.__super__.cancel.apply(this);
        },
        
        toogleControls: function(enable) {          
            this.aceEditor.setReadOnly(!enable);

            this.$field.find('.save-options button').prop("disabled", !enable);
            
            if (!enable) {
                this.$field.addClass('working');
            } else {
                this.$field.removeClass('working');
            }
        }
    });

    RAVEN.InlineEdit.AssigneePickerField = RAVEN.InlineEdit.SelectField.extend({
        initialize: function(elem, callbacks) {
            // call parent constructor
            RAVEN.InlineEdit.SelectPickerField.__super__.initialize.apply(this, [elem, callbacks]);

            // create JIRA select picker
            var $selectInput = this.$field.find('.field-control select');
            this.picker = this._createSelectPicker($selectInput);

            this.$field.find("strong").attr("rel",this.$field.attr("data-field-value"));
            this.$field.find("strong").addClass("user-hover");

            // do not propagate click event on suggestions
            var $select = this.$field.find('.field-edit select');
            $(document).on("click.inline-edit", '#' + $select.attr('id') + '-suggestions', function(e) {
                e.stopPropagation();
            });
        },

        setValue: function(value) {
            this.value = value;

            this.$field.find('.field-edit select').val(value);
            var $selectedOption = this.$field.find('.field-edit select option:selected');
            if ($selectedOption.length > 0) {
                var selected = new AJS.ItemDescriptor({
                    value: $selectedOption.val(),
                    label: $selectedOption.attr('label'),
                    selected: 'selected'
                });
                this.picker.setSelection(selected);
            }

            this.renderValue(value);
        },

        revertPicker: function() {
            // remove old picker
            var $el = this.picker.$container;
            $el.remove();

            var $selectInput = this.$field.find('.field-control select');
            this.picker = this._createSelectPicker($selectInput);
        },

        _createSelectPicker: function($selectField) {
            return new AJS.SingleSelect({
                element: $selectField,
                itemAttrDisplayed: "label",
                showDropdownButton: true,
                removeOnUnSelect: false,
                submitInputVal: true,
                revertOnInvalid: true,
                errorMessage: AJS.I18n.getText("user.picker.invalid.user", "'{0}'"),
                ajaxOptions: {
                    url: contextPath + "/rest/api/2/user/picker",
                    query: true, // keep going back to the sever for each keystroke
                    data: {
                        showAvatar: true
                    },
                    formatResponse: function(data) {
                        var groupDescriptor = new AJS.GroupDescriptor({
                            label: data.header
                        });
                        $(data.users).each(function(){
                            groupDescriptor.addItem(new AJS.ItemDescriptor({
                                value: this.key, // value of item added to select
                                label: this.displayName, // title of lozenge
                                html: this.html,
                                icon: this.avatarUrl,
                                allowDuplicate: false,
                                highlighted: true
                            }));
                        });
                        return [groupDescriptor];
                    }
                }
            });
        },

        getSubmitValue: function() {
            var optionbyId = this.$field.data('field-option-id');
            var value = this.getEditingValue();
            if (optionbyId && value) {
                return { 'id' : value };
            } else {
                return value;
            }
        },

        renderValue: function(value) {
            var label = this.$field.find('.field-edit select option:selected').text().split(" -")[0];
            if (!label) {
                label = SELECT_FIELD_EMPTY_ASSIGNEE;
            }

            this.$field.find('.field-view .field-content').empty().append('<strong class="user-hover" rel="' + value + '">' + label  + '</strong>');
        },

        edit: function() {
            this.$field.find('input').val(this.$field.find('input').val().split(" -")[0]);
            RAVEN.InlineEdit.SelectPickerField.__super__.edit.apply(this);
            this.$field.find('input').focus();
        },

        cancel: function() {
            RAVEN.InlineEdit.SelectPickerField.__super__.cancel.apply(this);

            // select the old value
            var $selectedOption = this.$field.find('.field-edit select option:selected');
            if ($selectedOption.length > 0) {
                var selected = new AJS.ItemDescriptor({
                    value: $selectedOption.val(),
                    label: $selectedOption.attr('label'),
                    selected: 'selected'
                });
                this.picker.setSelection(selected);
            }
        }
    });

})(AJS.$);
