(function($) {
	
	function formSubmitListener() {
		prepareAvailableIds();
		prepareRequirementIds();
		prepareDefectIds();

		return true;
	}

	function prepareAvailableIds() {
		var ids = $('#available-issue-types li').map(function() { 
			return $(this).data('id');
		}).get();
		
		$('#availableIssueTypeIds').val(JSON.stringify(ids));
	}

	function prepareRequirementIds() {
		var ids = $('#requirement-issue-types li').map(function() { 
			return $(this).data('id');
		}).get();

		$('#requirementIssueTypeIds').val(JSON.stringify(ids));
	}

	function prepareDefectIds() {
		var ids = $('#defect-issue-types li').map(function() { 
			return $(this).data('id');
		}).get();

		$('#defectIssueTypeIds').val(JSON.stringify(ids));
	}
	
	function onSubRequirementSubTaskChanged() {
		var subTasksChecked = $('#raven-subrequirement-tasks-condition').is(':checked');
		if (subTasksChecked === true) {
			$('#raven-issue-link-type').attr('disabled', 'disabled');
			$('#raven-issue-link-type-direction').attr('disabled', 'disabled');
		} else {
			$('#raven-issue-link-type').removeAttr('disabled');

			var value = $('#raven-issue-link-type').val();
			if (!value || value === 'NONE') {
				$('#raven-issue-link-type-direction').attr('disabled', 'disabled');
			} else {
				$('#raven-issue-link-type-direction').removeAttr('disabled');				
			}
		}
	}
	
	function onSubRequirementLinkTypeChanged() {
		var value = $('#raven-issue-link-type').val();

		// remove all previous options
		$('#raven-issue-link-type-direction').empty();			
		
		if (value && value !== 'NONE') {
			$('#raven-issue-link-type-direction').removeAttr('disabled');

			var option = $('#raven-issue-link-type :selected')[0];
			if (option) {
				var $op = $(option);
				var valueOutIn = $op.data('outward') + ' -> ' + $op.data('inward');
				var valueInOut = $op.data('inward') + ' -> ' + $op.data('outward');
			
				$('#raven-issue-link-type-direction')
					.append('<option value="outward">' + valueOutIn + '</option>')
					.append('<option value="inward">' + valueInOut + '</option>');
			}
		} else {
			$('#raven-issue-link-type-direction').append('<option value="NONE" selected>'+AJS.I18n.getText('issue-type-mapping-configuration.label.select-direction')+'...</option>');
			$('#raven-issue-link-type-direction').attr('disabled', 'disabled');
		}
	}
	
	function initIssueTypeMappingConfig() {
		onSubRequirementSubTaskChanged();
							
		$(document).on('change', '#raven-subrequirement-tasks-condition', function(event) {
			onSubRequirementSubTaskChanged();
		});
		
		$(document).on('change', '#raven-issue-link-type', function(event) {
			onSubRequirementLinkTypeChanged();
		});

        $(document).on('click', '.issue-type-remove', function(event) {
            $(this).closest('li').remove();
        });
	}

	$(document).ready( function() {
		AJS.messages.setup();

		initIssueTypeMappingConfig();

		var form = document.getElementById('issue-type-mapping-form');
		if (form.attachEvent) {
			form.attachEvent("onsubmit", formSubmitListener);
		} else {
			form.addEventListener("submit", formSubmitListener);
		}

		$("#available-issue-types").sortable({
			connectWith: ".issue-type-list.receiver",
            forcePlaceholderSize: true,
            helper: function(e,li) {
                copyHelper = li.clone().insertAfter(li);
                return li.clone();
            },
            stop: function() {
                copyHelper && copyHelper.remove();
            }
		}).disableSelection();

        $(".issue-type-list.receiver").sortable({
            receive: function(e,ui) {
                // cancel if already present
                var list = ui.item.closest('.issue-type-list');
                if (list.find('li[data-id = ' + ui.item.data('id') + ']').length > 1) {
                    // remove item instead of cancelling
                    $(ui.item).remove();
                } else {
                    $(ui.item).find('span.draghandle').remove();
                    $(ui.item).find('.issue-type-item').append('<span class="aui-icon aui-icon-small aui-iconfont-remove issue-type-remove">'+AJS.I18n.getText('issue-type-mapping-configuration.label.select-direction')+'</span>');
                }
                copyHelper = null;
            }
        });

		$('.aui-message.success').delay(5000).fadeOut('slow');
	});
})(AJS.$);
