(function($) {

    var initContext = null;

    var __$ = function(elem) {
        return getContext().find(elem);
    }

    function getContext() {
        if (initContext) {
            return $(initContext);
        } else {
            var $context = $('#create-issue-dialog:visible, #edit-issue-dialog:visible, #create-test-issue-dialog:visible, \
                              #create-issue-dialog.jira-dialog-open, #create-issue-dialog.aui-dialog-open');
            if ($context.length > 0) {
                return $($context);
            } else {
                return $('#page');
            }
        }
    }

    String.prototype.bool = function() {
        return (/^true$/i).test(this);
    };

    function getConditionIssueKey() {
        return __$('#raven-precondition-field-type-issue-key').val();
    }

    function hasTests() {
        return __$('#raven-has-tests').val() === 'true';
    }

    function getPersistedPreConditionTypeValue() {
        return __$('#raven-precondition-field-type-value').val();
    }

    function isManualTestTypeOption(value) {
        var manualOptions = __$("#manual-options").val();
        if (manualOptions) {
            var ids = JSON.parse(manualOptions);
            return _.contains(ids, parseInt(value, 10));
        }

        return false;
    }

    function isCucumberTestTypeOption(value) {
        var cucumberOptions = __$("#cucumber-options").val();
        if (cucumberOptions) {
            var ids = JSON.parse(cucumberOptions);
            return _.contains(ids, parseInt(value, 10));
        }

        return false;
    }

    function isOtherTestTypeOption(value) {
        return isCucumberTestTypeOption(value) == false && isManualTestTypeOption(value) == false;
    }

    function compareTypeOptionValues(value1, value2) {
        return (isManualTestTypeOption(value1) === true && isManualTestTypeOption(value2) === true) ||
            (isCucumberTestTypeOption(value1) === true && isCucumberTestTypeOption(value2) === true) ||
            (value1 == value2);
    }

    function performChangeTestType() {
        var typeFieldId = __$('#raven-precondition-field-type').val();

        var typeField = __$('#' + typeFieldId);
        var conditionField = __$('#precondition-condition-create-ace-editor-hidden');
        var manualField = __$('#precondition-manual-id');

        var previousValue = typeField[0].previousValue;
        if (isManualTestTypeOption(previousValue) === true || isOtherTestTypeOption(previousValue)) {
            applyConditionToAllTypes(__$('#precondition-manual-id').val());
        } else {
            var preconditionConditionAceEditor = getCurrentAceEditor();
            if (preconditionConditionAceEditor) {
                applyConditionToAllTypes(preconditionConditionAceEditor.getSession().getValue());
            }
        }

        var value = typeField.val();
        var currentType = $.trim(typeField.children('option[value=' + value + ']').text());
        if (isManualTestTypeOption(value) === true || isOtherTestTypeOption(value)) {
            conditionField.parent().hide();
            manualField.parent().show();
            var conditionFieldId = __$('#precondition-condition-create-ace-editor-hidden').attr('name');
            __$('#precondition-condition-create-ace-editor-hidden').attr('name', conditionFieldId + '_zombie');
            __$('#precondition-condition-create-ace-editor-hidden').val('');
            __$('.ace_cursor').css('left', '4px');
            setTimeout(function() {
                manualField.focus();
            }, 0);
            __$('#precondition-manual-wiki-edit').next().text(AJS.I18n.getText('precondition-editor-custom-field.label.text'));

            // if test key is empty then this is the create screen and we don't need to reset stspes field in the server
        } else {
            manualField.parent().hide();
            conditionField.attr('name', manualField.attr('name'));
            __$('#precondition-manual-wiki-edit').next().text(AJS.I18n.getText('precondition-editor-custom-field.label.cucumber'));
            conditionField.parent().show();
        }
        __$('#precondition-test-match-type').val(currentType);
        typeField[0].previousValue = typeField.val();

        // show PreCondition -> Test type mismatch warning
        var typeMatches = compareTypeOptionValues(getPersistedPreConditionTypeValue(), typeField.val());
        if (hasTests() && typeMatches === true) {
            __$('#raven-edit-precondition-type-test-mismatch-message').hide();
        } else {
            __$('#raven-edit-precondition-type-test-mismatch-message').show();
        }
    }

    function applyConditionToAllTypes(value) {
        __$('#precondition-condition-hidden-value').val(value);
        __$('#precondition-condition-hidden-new-value').val(value);
        __$('#precondition-condition-create-ace-editor-hidden').val(value);
        __$('#precondition-manual-id').val(value);

        var preconditionConditionAceEditor = getCurrentAceEditor();
        if (preconditionConditionAceEditor) {
            preconditionConditionAceEditor.getSession().setValue(value);
        }
    }

    function getCurrentAceEditor() {
        return __$('#precondition-condition-create-ace-editor').data('ace-editor');
    }

    function initScreen() {
        var typeFieldId = __$('#raven-precondition-field-type').val();

        var select;
        if (typeFieldId) {
            select = __$('#' + typeFieldId)[0];
            if (!select) {
                return;
            }

            select.previousValue = select.value;
            performChangeTestType();

            var $preCondTypeSelect = __$('#' + __$('#raven-precondition-field-type').val());
            $preCondTypeSelect.off('change.raven-edit-precondition');
            $preCondTypeSelect.on('change.raven-edit-precondition', function() {
                performChangeTestType();
            });
        }

        // init ace gherkin editor
        var editorElem = __$('#precondition-condition-create-ace-editor');
        if (editorElem.length > 0) {
            require(['ace/ace'], function(ace) {
                var editor = ace.edit(editorElem[0]);
                var theme = require("ace/theme/textmate");
                editor.setTheme(theme);
                editor.setFontSize(14);
                editor.getSession().setMode("ace/mode/gherkin");
                editor.getSession().setUseWrapMode(false)
                editor.renderer.setShowGutter(true);
                editor.renderer.setShowPrintMargin(false);

                editor.getSession().setValue(__$('#precondition-condition-create-ace-editor-hidden').val());

                editor.getSession().on('change', function() {
                    __$('#precondition-condition-create-ace-editor-hidden').val(editor.getSession().getValue());
                });

                // register dom reference
                editorElem.data('ace-editor', editor);
            });
        }
    }

    // register for NEW_CONTENT_ADDED
    JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, function(e, context, reason) {
        if (context) {
            initContext = context.parent();
            initScreen();
            initContext = null;
        } else {
            initScreen();
        }
    });

    /* Sometimes, the JIRA.CONTENT_ADDED_REASON.pageLoad event fires before the
     * page is completly loaded and the ace global object is not yet available.
     * Because of this, we init the screen (with the ace editor) only on dom ready.
     */
    $(document).ready(function() {
        initScreen();
    });

})(AJS.$);