(function($) {

    RAVEN.namespace("RAVEN.DataTable");

    /**
     * This is the main view for the Test Runs table on a Test Execution issue
     */
    RAVEN.DataTable.BulkEditView = Backbone.View.extend({
        /**
         * Indicates if the table is in Bulk mode
         */
        isBulk: false,

        events: {
            'click #bulk-edit-action': '_onBulkToogleClick',
            'click .data-table #cbSelectAll': '_onBulkSelectAllClick',
            'click .data-table tbody tr .bulkEdit': '_onBulkSelectClick'
        },

        initialize: function() {
            this.render();
        },

        render: function() {
            // if filter already exists then return
            if (this.$el.find('#bulk-edit-action').length > 0) {
                return;
            }

            this.$el.prepend('\
                <div id="bulk-edit-toogle" class="bulk-mode-button-wrapper">\
                <button id="bulk-edit-action" class="aui-button aui-button-subtle noselect" title="' + AJS.I18n.getText('common.view-issue-section.bulk-mode-button.title') + '">\
                    <span class="bulk-edit-action-span"></span>\
                </button>\
                </div>');

            this.$el.addClass('datatable-bulk-support');

            return this;
        },

        /**
         * Show or hide the bulk commands 
         */
        showHideBulkActions: function(e) {
            if(e !== undefined) {
                if (this.isBulk === true) {
                    $(e.target).closest('.raven-data-table-wrapper').find('.singleEdit').hide();
                    $(e.target).closest('.raven-data-table-wrapper').find('.bulkEdit').show();
                } else {
                    $(e.target).closest('.raven-data-table-wrapper').find('.singleEdit').show();
                    $(e.target).closest('.raven-data-table-wrapper').find('.bulkEdit').hide();
                }
            }else {
                if (this.isBulk === true) {
                    this.$el.closest('.raven-data-table-wrapper').find('.singleEdit').hide();
                    this.$el.closest('.raven-data-table-wrapper').find('.bulkEdit').show();
                } else {
                    this.$el.closest('.raven-data-table-wrapper').find('.singleEdit').show();
                    this.$el.closest('.raven-data-table-wrapper').find('.bulkEdit').hide();
                }
            }
        },
            
        /**
         * Get the selected row ids
         */
        getSelectedRowIds: function() {
            var testKeys = [];

            this.$el.find("table tbody tr.checked").each(function() {
                testKeys.push($(this).attr('id'));
            });
                            
            return testKeys;
        },

        /**
         * Get the selected rows whatever it is inline action or not
         */
        getSelectedRows: function() {
            return this.$el.find("table tbody tr.checked");
        },

        /**
         * Set checkBox with value if checkBoxId is null then we set all checkboxes from datatable
         */
        setCheckBoxRowsSelectionValue: function(checked, checkBoxId) {
            
            //Check if we are dealing with one row or all rows
            if (checkBoxId != null && checkBoxId.length > 0){
                checkBoxId = ' #' + checkBoxId;
            } else {
                checkBoxId=' :checkbox';
                
                this.$el.find('.data-table #cbSelectAll').prop('checked', checked);
            }
            
            var $checkInput = this.$el.find('.data-table tbody tr '+ checkBoxId);
            if (checked) {
                $checkInput.prop('checked', true);
                $checkInput.closest('tr').addClass('checked');
            } else {
                $checkInput.prop('checked', false);
                $checkInput.closest('tr').removeClass('checked');
            }
        },

        /**
         * Set listener to bulk mode button
         */
        _onBulkToogleClick: function(e) {
            $(e.target).closest('.raven-data-table-wrapper').find('#bulk-edit-action').toggleClass('active');
            this.isBulk = !this.isBulk;
            //show or hide the bulk commands 
            this.showHideBulkActions(e);

            if (!this.isBulk) {
                this.setCheckBoxRowsSelectionValue(false, null);
            }
        },

        /**
         * Selection of all rows for bulk change
         */
        _onBulkSelectAllClick: function(e) {
            this.setCheckBoxRowsSelectionValue($(e.target).is(':checked'), null);
        },

        /**
         * Selection of multiple rows for bulk operation
         */
        _onBulkSelectClick: function(e) {
            var $checkInput = $(e.target);
            if ($checkInput.is(':checked')) {
                $checkInput.closest('tr').addClass('checked');
            } else {
                $checkInput.closest('tr').removeClass('checked');
            }
            
            var numberSelectBox = $(".data-table tbody tr .bulkEdit").length;
            var numberSelected = $(".data-table tbody tr.checked .bulkEdit").length;
            if(numberSelectBox == numberSelected) {
            	$(".data-table thead th #cbSelectAll").prop("checked",true);
            } else {
            	$(".data-table thead th #cbSelectAll").prop("checked",false);
            }
        }
    });
})(AJS.$);