define('js/field/subtle-single-select', [
    'jira/ajs/select/single-select',
    'underscore',
    'jquery'
], function(
    SingleSelect,
    _,
    $
) {

    /**
     * Generic subtle single select picker dropdown implementation
     */
    var SubtleSingleSelect = function (items, defaultItem, $container) {
        this.items = items;
        this.defaultItem = defaultItem;
        this.$container = $container;
        this.selectedItem = undefined;
    };

    /**
     * Set the items
     */
    SubtleSingleSelect.prototype.setItems = function (items) {
        this.items = items || [];
    };

    /**
     * Get the items
     */
    SubtleSingleSelect.prototype.getItems = function () {
        return this.items;
    };

    /**
     * Are there any items
     */
    SubtleSingleSelect.prototype.hasItems = function () {
        return !_.isEmpty(this.items);
    };

    SubtleSingleSelect.prototype.getDefaultItem = function () {
        return this.defaultItem;
    };
    SubtleSingleSelect.prototype.getDefaultItemId = function () {
        return this.defaultItem ? this.defaultItem.id : false;
    };

    /**
     * Set the default item to load when no default has been set
     * @param defaultItem an object representing one of the items in the list
     */
    SubtleSingleSelect.prototype.setDefaultItem = function (defaultItem) {
        this.defaultItem = defaultItem || undefined;
    };

    /**
     * Get an item given its id
     */
    SubtleSingleSelect.prototype.getItemById = function (itemId) {
        var item = _.find(this.items, function (item) {
            return item.id.toString() === itemId.toString();
        });
        return _.isUndefined(item) ? false : item;
    };

    /**
     * Get the currently selected item. This will automatically select the first active item if current selection
     * is invalid.
     * @return a object representing the item or false if no items are available
     */
    SubtleSingleSelect.prototype.getSelectedItem = function () {
        var item = this.selectedItem;

        if (!item) {
            // select the default item if we have one
            if (this.getDefaultItem()) {
                item = this.getDefaultItem();
            } else if (this.hasItems()) {
                // fall back to last item
                item = _.last(this.getItems());
            }
        }

        return item || false;
    };

    /**
     * Set the new selected item
     */
    SubtleSingleSelect.prototype.setSelectedItem = function (selectedItemId, preventEventTrigger) {
        this.previousSelectedItem = this.getSelectedItem();
        // fetch the item for the given id
        var item = this.getItemById(selectedItemId);

        this.selectedItem = item;

        if (!preventEventTrigger) {
            $(this).trigger('selectedItemChanged', {item: item});
        }
    };

    //
    // Rendering logic
    //
    SubtleSingleSelect.prototype.render = function () {
        // after the frother control is disabled, we want to redraw this
        var self = this;
        var onSelect = function (event, selected) {
            self.setSelectedItem(selected.value());
        };

        var showDropdownAndRedraw = function (e) {
            self.showDropdown(e, function () {
                self.render(self.$container);
            }, onSelect);
        };

        var selectedItem = this.getSelectedItem();

        this.$container.html(RAVEN.SubtleSingleSelect.render({
            items: this.items,
            selectedItem: selectedItem
        }));
        this.$container.find('#raven-subtle-items-trigger').click(showDropdownAndRedraw);

        this.$container.find('#raven-subtle-picker-form').submit(function () {
            return false;
        });
    };

    SubtleSingleSelect.prototype.showDropdown = function (e, redrawFn, selectFn) {
        $(e.target).hide();

        var self = this;
        var $select = this.$container.find('#raven-subtle-picker');
        var singleSelect = new SingleSelect({
            element: $select,
            width: 200,
            errorMessage: '',
            setMaxHeightToWindow: true
        });

        var $field = this.$container.find('#raven-subtle-picker-field');

        $select.bind('selected', function (e, selected) {
            if (selected.value()) {
                selectFn(e, selected);
            }
            $field.blur();
        });

        $field.blur(redrawFn);

        $field.focus().select();

        // hack :( but I cannot find another way to force the frother control to show the suggestions
        // this seems to be the shortest timeout that works reliably
        setTimeout(function () {
            self.$container.find('#raven-subtle-picker-field').trigger('click');
        }, 20);

    };

    return SubtleSingleSelect;
});

AJS.namespace('RAVEN.Field.SubtleSingleSelect', null, require('js/field/subtle-single-select'));
