(function($) {
	RAVEN.namespace("RAVEN.Gadget.fields");

    function message(title, message, type) {
        clearMessage();
        if (!type) {
            type = 'error';
        }
        AJS.messages[type]($('#gadget-message'), {
            title: title,
            body: message
        });
    }

    function clearMessage() {
        $('#gadget-message').empty();
    }

	function fetchProjectGroupingFields(gadget, project) {
		if (project) {
			$("#grouping").attr('disabled', 'disabled');
			$("#grouping").empty(); // clear options
			$.ajax({
				type: "GET",
	    		url: gadget.getBaseUrl() + "/rest/raven/1.0/chart/requirement/project/" + project + "/groupingFields",
	    		contentType: "application/json",
	    		success: function(data) {
	    			if (data && data.length > 0) {
                        var nativeFields = $(data).filter(function () {
                            return this.id && this.id.lastIndexOf('customfield_', 0) !== 0;
                        });

                        var customFields = $(data).filter(function () {
                            return this.id && this.id.lastIndexOf('customfield_', 0) === 0;
                        });

                        var $nativeGroup = $('<optgroup label="Native"></optgroup>');
	    				$.each(nativeFields, function(i, v) {
	    					var selected = "";
	    					if (gadget && gadget.getPref("grouping") === v.id) {
	    						selected = " selected";
	    					}
                            
	    					$nativeGroup.append('<option value="' + v.id + '"' + selected + '>' + v.name + '</option>');
	    				});
                        $("#grouping").append($nativeGroup);

                        if (customFields && customFields.length > 0) {
                            var $cfGroup = $('<optgroup label="Custom Field"></optgroup>');
                            $.each(customFields, function(i, v) {
                                var selected = "";
                                if (gadget && gadget.getPref("grouping") === v.id) {
                                    selected = " selected";
                                }
                                
                                $cfGroup.append('<option value="' + v.id + '"' + selected + '>' + v.name + '</option>');
                            });
                            $("#grouping").append($cfGroup);
                        }
	    			}
	    		},
				error: function(qXHR, textStatus, errorThrown) {
					if (errorThrown && errorThrown.trim() != '') {
						errorThrown = errorThrown + ': ' + qXHR.responseText;
					} else {
						errorThrown = qXHR.responseText;
					}
					if (gadget) {
						message(gadget.getMsg("gadget.requirement-coverage.common.error.groupings"), errorThrown);
					} else {
						message(null, errorThrown);
					}
				},
				complete: function() {
					$("#grouping").removeAttr('disabled');
				}
			});
		}
	}

	RAVEN.Gadget.fields.GroupingSelect = function(gadget, groupingUserPref, selectedProject) {
		return {
			id: "grouping-callback-field",
        	userpref: groupingUserPref,
			label: "Group by",
			description: gadget.getMsg("gadget.requirement-coverage.common.field.grouping.desc"),
			type: "callbackBuilder",
			callback: function(parentDiv) {
				var $el = $("<select></select>").attr({
					"id": "grouping",
					"class": "select",
					"name": "grouping"
				});
				parentDiv.append($el);
				var desc = gadget.getMsg("gadget.requirement-coverage.common.field.grouping.desc");
				parentDiv.append('<div class="description">' + desc + '</div>');

				var project = null;
				if (typeof(selectedProject) == "function") {
					project = selectedProject();
				} else if (typeof selectedProject == 'string' || selectedProject instanceof String) {
					project = selectedProject;
				}
				if (project) {
					fetchProjectGroupingFields(gadget, project);
				}
			},
			initProjectGroupings: function(gadget, project) {
				fetchProjectGroupingFields(gadget, project);		
			}
		};
	}

    function fetchTestRunGroupingFields(gadget) {
        $("#grouping").attr('disabled', 'disabled');
        $("#grouping").empty(); // clear options
        $.ajax({
            type: "GET",
            url: gadget.getBaseUrl() + "/rest/raven/1.0/chart/testrun/groupingFields",
            contentType: "application/json",
            success: function (data) {
                if (data && data.length > 0) {
                    var testFields = $(data).filter(function () {
                        return this.id && this.id.lastIndexOf('testrun_', 0) !== 0;
                    });

                    var testrunFields = $(data).filter(function () {
                        return this.id && this.id.lastIndexOf('testrun_', 0) === 0;
                    });

                    var $testGroup = $('<optgroup label="Test Field"></optgroup>');
                    $.each(testFields, function (i, v) {
                        var selected = "";
                        if (gadget && gadget.getPref("grouping") === v.id) {
                            selected = " selected";
                        }

                        $testGroup.append('<option value="' + v.id + '"' + selected + '>' + v.name + '</option>');
                    });
                    $("#grouping").append($testGroup);

                    if (testrunFields && testrunFields.length > 0) {
                        var $cfGroup = $('<optgroup label="Test Run Field"></optgroup>');
                        $.each(testrunFields, function (i, v) {
                            var selected = "";
                            if (gadget && gadget.getPref("grouping") === v.id) {
                                selected = " selected";
                            }

                            $cfGroup.append('<option value="' + v.id + '"' + selected + '>' + v.name + '</option>');
                        });
                        $("#grouping").append($cfGroup);
                    }
                }
            },
            error: function (qXHR, textStatus, errorThrown) {
                if (errorThrown && errorThrown.trim() != '') {
                    errorThrown = errorThrown + ': ' + qXHR.responseText;
                } else {
                    errorThrown = qXHR.responseText;
                }
                if (gadget) {
                    message(gadget.getMsg("gadget.test-runs-summary.common.error.groupings"), errorThrown);
                } else {
                    message(null, errorThrown);
                }
            },
            complete: function () {
                $("#grouping").removeAttr('disabled');
            }
        });
    }

    RAVEN.Gadget.fields.TestRunGroupingSelect = function(gadget, groupingUserPref) {
        return {
            id: "grouping-callback-field",
            userpref: groupingUserPref,
            label: gadget.getMsg("gadget.test-runs-summary.common.field.grouping.label") + ":",
            description: gadget.getMsg("gadget.test-runs-summary.common.field.grouping.desc"),
            type: "callbackBuilder",
            callback: function(parentDiv) {
                var $el = $("<select></select>").attr({
                    "id": "grouping",
                    "class": "select",
                    "name": "grouping"
                });
                parentDiv.append($el);
                var desc = gadget.getMsg("gadget.test-runs-summary.common.field.grouping.desc");
                parentDiv.append('<div class="description">' + desc + '</div>');

                fetchTestRunGroupingFields(gadget);
            },
            initGroupings: function(gadget) {
                fetchTestRunGroupingFields(gadget);
            }
        };
    }
})(AJS.$);