(function($) {

	RAVEN.namespace("RAVEN.Gadget");

	var historicalChart = null;

	function message(title, message, type) {
		clearMessage();
		if (!type) {
			type = 'error';
		}
		AJS.messages[type]($('#gadget-message'), {
			title: title,
			body: message
		});
	}

    function serverError(gadget, message) {
        var obj = null;
        try {
            obj = JSON.parse(message);
        } catch(e) {}
        
        var msg = null;
        if (obj && obj.message) {
            msg = obj.message;
        } else {
            msg = gadget.getMsg('gadget.requirement-coverage.common.error.data');
        }

        $('.gadget .view').append('<div id="gadget-message" style="margin:10px;"></div>');
        AJS.messages['error']($('#gadget-message'), {
            body: msg
        });
    }
	
	function clearMessage() {
		$('#gadget-message').empty();
	}

	function createHistoricalCoverageChart(gadget, data) {
		var content = $('<div id="gadget-content" style="margin: 5px;"><div id="gadget-message"></div><div id="historical-requirement-coverage-chart"></div></div>');
		gadget.getView().html(content);

		if (data && data.resultset && data.resultset.length > 0) {
			clearMessage();
			
			var type = gadget.getPref('chartType');

			var options = {
				canvas: "historical-requirement-coverage-chart",
				width: gadgets.window.getViewportDimensions().width - 20,
				height: 400,
				stacked: true,
				animate: true,
				legend:  true,
				selectable: false,
				clickable: true,
				hoverable: true,
				legendPosition: 'top',
				legendAlign:    'left',
				legendFont : '12px sans-serif',
				valuesFont : 'bold 12px sans-serif',
				baseAxisTicks: true,
				valuesVisible: true,
				valuesOptimizeLegibility: true,
				ignoreNulls: false,
				
				colors: createColors(data.dimensions),
				
				extensionPoints: {
					xAxisLabel_textAngle:    -Math.PI/3,
					xAxisLabel_textAlign:    'right',
					xAxisLabel_textBaseline: 'top',
					barLabel_text: function() {
						// don't show percentages less than 5% because the label is too big... 
						var categoriesCount = this.chart.data.getVisibleCategories().length;
						if (categoriesCount < 7) { 
							var percent = this.scene.vars.value.percent.value;
							if (percent > 0.05) {
								if (type === 'absolute') {
									return this.scene.vars.value.value;
								} else {
									return this.scene.vars.value.percent.label;
								}
							}
						}
						return null;
					}
				}
			};
			
			var historicChart;
			if (type === 'relative') {
				historicChart = new pvc.NormalizedBarChart(options);
			} else {
				historicChart = new pvc.BarChart(options);
			}
			
			historicChart.setData(data, {crosstabMode: false });
			historicChart.render();
		} else {
			message(null, gadget.getMsg("gadget.requirement-coverage.common.empty"), 'info');
		}
	}

    function createColors(dimensions) {
		if (dimensions === 3) {
			return [
				'#A4BFD6', '#3671A3', '#1F426C', // Uncovered
				'#F9F1BB', '#F1E069', '#CAB63D', // Not run
				'#D1D5D8', '#9AA1A8', '#616B73', // Unknown
				'#ECB6B1', '#D45D52', '#A73730', // NOK
				'#CFE3B7', '#95C160', '#5C9038' // OK
			];
		} else if (dimensions === 2) {
			return [
				'#A4BFD6', '#3671A3', // Uncovered
				'#F9F1BB', '#F1E069', // Not run
				'#D1D5D8', '#9AA1A8', // Unknown
				'#ECB6B1', '#D45D52', // NOK
				'#CFE3B7', '#95C160' // OK
			];
		} else if (dimensions === 1) {
			return [
				'#A4BFD6', // Uncovered
				'#F9F1BB', // Not run
				'#D1D5D8', // Unknown
				'#ECB6B1', // NOK
				'#CFE3B7', // OK
			];
		} else {
			return [];
		}
    }

	function setTitle(gadget) {
		var execScope = gadget.getPref("execScope");
        var filterId = gadget.getPref("filterId");
        var project = gadget.getPref("project");
        var projectFilter = gadget.getPref("projectFilter");
        var testPlan = gadget.getPref("testPlanKey");
        var version = gadget.getPref("version");

        var params = '?execScope=' + execScope;

        if (project && project != '__ALL__') {
            params = params + '&project=' + project;
        }

        if (projectFilter) {
            params = params + '&projectFilter=' + projectFilter;
        }

        if (version && version != -1) {
            params = params + '&version=' + version;
        }

        if (testPlan && testPlan != -1) {
            params = params + '&testPlan=' + testPlan;
        }

        if (filterId) {
            // remove prefix "filter-"
            params = params + "&savedFilter=" + filterId.substring(7);
        }

		$.ajax({
			type: "GET",
			url: gadget.getBaseUrl() + '/rest/raven/1.0/chart/requirement/title' + params,
			contentType: "application/json",
			success: function(data) {
				if (data && data.titleParams) {
					var title = AJS.format(gadget.getMsg("gadget.requirement-coverage.historical.title"), data.titleParams);
					gadgets.window.setTitle(title);
				} else {
					var title = gadget.getMsg("gadget.requirement-coverage.historical.title.default");
					gadgets.window.setTitle(title);
				}
			}
		});
	}

	RAVEN.Gadget.historicalRequirementCoverage = function(contextPath) {
		return AJS.Gadget({
			baseUrl: contextPath, /* Used to make base url available to gadget */
			useOauth: "/rest/gadget/1.0/currentUser",
			config: {
    			descriptor: function(args) {
					var gadget = this;

					// register event handler for refreshing project versions
					$(document).off('change.raven').on('change.raven', '#project', function() {
						var groupingField = gadget.groupingField;
                        var versionField = gadget.versionField;
						var selectedProject = $("#project option:selected").val();

                        groupingField.initProjectGroupings(gadget, selectedProject);
						versionField.initProjectVersions(gadget, selectedProject);
					});

					// register event handler for toggling execution scope fields
					$(document).on('change.raven.execscope', '#execScope', function() {
						var selectedScope = $("#execScope option:selected").val();
						if (selectedScope === "version") {
							$("#testPlanKey").prop('disabled', true);
							$("#version").prop('disabled', false);
						} else {
							$("#testPlanKey").prop('disabled', false);
							$("#version").prop('disabled', true);
						}
					});

					var versionDisabled = false;
					var testPlanKeyDisabled = false;
					var selectedScope = gadget.getPref("execScope");
					if (selectedScope === "version") {
						testPlanKeyDisabled = true;
					} else {
						versionDisabled = true;
					}

					// get project keys because the versions rest service only 
					// accepts keys for project in JIRA prior to v6.1 
					var projects = $.map(args.projects, function(p) {
						return {
							label: p.name,
							value: p.key
						};
					});

                    projects = [{
                        label: gadget.getMsg("common.label.all"),
                        value: "__ALL__"
                    }].concat(projects);

                    gadget.groupingField = RAVEN.Gadget.fields.GroupingSelect(gadget, "grouping", function() {
                        return $("#project option:selected").val(); //gadget.getPref("project");
                    });

					gadget.versionField = RAVEN.Gadget.fields.VersionSelect(gadget, "version", versionDisabled, true, function() {
						return $("#project option:selected").val(); //gadget.getPref("project");
					});

                    var filters = null;
                    $.each(args.filters.options, function() {
                        if(this.group.label === "Filters") {
                            filters = this.group;
                        }
                    });
                    
                    if (filters) {
                        var filterId = gadget.getPref("filterId");
                        if (filterId) {
                            var filter = _.find(filters.options, function(f) {
                                return f.value == filterId;
                            });
                            if (filter) {
                                gadget.projectOrFilterName = filter.label;
                            }
                        } else {
                            gadget.projectOrFilterName = null;
                        }
                    } else {
                        gadget.projectOrFilterName = null;
                    }

                    var searchParam;
                    if (/^jql-/.test(this.getPref("filterId")) || this.getPref("isPopup") === "true") {
                        searchParam =
                            {
                                userpref: "filterId",
                                type: "hidden",
                                value: gadgets.util.unescapeString(gadget.getPref("filterId"))
                            };
                    }
                    else {
                        searchParam = RAVEN.gadget.fields.filterPicker(gadget, "filterId");
                    }

					var numberDays = gadget.getPref("numberDays");
					if (!numberDays) {
						numberDays = args.settings.defaultDays;
					}

					return {
					    action: '/rest/raven/1.0/chart/requirement/historical/validate',
					    theme : function() {
					        if (gadgets.window.getViewportDimensions().width < 450){
					            return "gdt top-label";
					        } else {
					            return "gdt";
					        }
					    }(),
					    fields: [
                            searchParam,
                            {
                                userpref: "projectFilter",
                                type: "checkbox",
                                options:[
                                    {
                                        id: "filter-project-issues",
                                        label: gadget.getMsg("gadget.requirement-coverage.common.field.project-filter.desc"),
                                        value: "true",
                                        selected : gadget.getPref("projectFilter") == 'true'
                                    }
                                ]
                            },
                            {
								id: "execScope",
        						userpref: "execScope",
        						label: "Execution Scope",
        						description: gadget.getMsg("gadget.requirement-coverage.common.field.exec-scope.desc"),
        						type: "select",
        						selected: gadget.getPref("execScope"),
        						options:[
						            {
						                label: "Version",
						                value: "version"
						            },
						            {
						                label: "Test Plan",
						                value: "testPlan"
						            }
						        ]
							},
					    	{
								userpref: "project",
								label: "Project",
								description: gadget.getMsg("gadget.requirement-coverage.common.field.project.desc"),
								type: "select",
								selected: gadget.getPref("project"),
								options: projects
							},
							{
                                id: "testPlanKey",
                                userpref: "testPlanKey",
                                label: "Test Plan Key",
                                description: gadget.getMsg("gadget.requirement-coverage.common.field.test-plan-key.desc"),
                                type: "custom",
                                value: gadget.getPref("testPlanKey"),
                                template: function() {
                                	var disabled = "";
                                	if (testPlanKeyDisabled === true) {
                                		disabled = " disabled"
                                	}
						            return '<input type="text" class="text" value="' + gadget.getPref("testPlanKey") + '" id="testPlanKey" name="testPlanKey"' + disabled + '>';
						        }
                            },
					        gadget.versionField,
                            gadget.groupingField,
                            {
                                userpref: "flattenRequirements",
                                type: "checkbox",
                                options:[
                                    {
                                        id: "flat-requirement-presentation",
                                        label: gadget.getMsg("gadget.requirement-coverage.common.field.flat-requirement-presentation.desc"),
                                        value: "true",
                                        selected : gadget.getPref("flattenRequirements") == 'true'
                                    }
                                ]
                            },
							{
                                id: "number-days",
                                userpref: "numberDays",
                                label: "Number of Days",
                                description: gadget.getMsg("gadget.requirement-coverage.historical.field.numdays.desc"),
                                type: "text",
                                value: gadget.getPref("numberDays")
                            },
							{
								id: "charttype",
        						userpref: "chartType",
        						label: "Chart Type",
        						description: gadget.getMsg("gadget.requirement-coverage.historical.field.chart-type.desc"),
        						type: "select",
        						selected: gadget.getPref("chartType"),
        						options:[
						            {
						                label: "absolute",
						                value: "absolute"
						            },
						            {
						                label: "relative",
						                value: "relative"
						            }
						        ]
							},
    						AJS.gadget.fields.nowConfigured()
					    ]
					};
				},
    			args: [
    			    {
        				key: "settings",
						ajaxOptions: "/rest/raven/1.0/chart/requirement/settings"
    				},
    				{
        				key: "projects",
						ajaxOptions: {
							url: "/rest/api/2/project",
							contentType: "application/json"
						}
    				},
                    {
                        key: "filters",
                        ajaxOptions: {
                            url: "/rest/gadget/1.0/filtersAndProjects",
                            contentType: "application/json"
                        }
                    }
				]
			},
		   	view: {
				enableReload: true,
				onResizeReload: true,
				onResizeAdjustHeight: false,
				template: function (args) {
					var gadget = this;

					// set gadget title
					setTitle(this);

					// server errors and timeout errors are not catched by the error handler of the REST date request !!!
					if (!args.data) {
						message(null, gadget.getMsg("gadget.requirement-coverage.common.error.data"));
					} else {
						createHistoricalCoverageChart(gadget, args.data);
					}
				},
				args: [{
					key: "data",
					ajaxOptions: function() {
                        var gadget = this;

                        var project = this.getPref("project");
                        var projectFilter = this.getPref("projectFilter");
                        var execScope = this.getPref("execScope");
                        var version = this.getPref("version");
                        var testPlan = this.getPref("testPlanKey");
                        var filterId = gadgets.util.unescapeString(this.getPref("filterId"));

                        var params = '';
						if (execScope) {
                            params = params + '&execScope=' + execScope;
                        }

                        if (project && project != '__ALL__') {
                            params = params + '&project=' + project;
                        }

                        if (projectFilter) {
                            params = params + '&projectFilter=' + projectFilter;
                        }

                        if (version && version != -1) {
                            params = params + '&version=' + version;
                        }

                        if (testPlan && testPlan != -1) {
                            params = params + '&testPlan=' + testPlan;
                        }

                        if (filterId) {
                            // remove prefix "filter-"
                            params = params + "&savedFilter=" + filterId.substring(7);
                        }

						return {
		    				type: "GET",
		    				url: contextPath 
		    						+ '/rest/raven/1.0/chart/requirement/historical'
                                    + '?groupingField=' + this.getPref("grouping")
                                    + '&flattenRequirements=' + this.getPref("flattenRequirements")
		    						+ '&numberDays=' + this.getPref("numberDays")
                                    + params,
		    				processData: false,
		    				async: true,
							timeout: 120000,
		    				contentType: "application/json",
							error: function(qXHR, textStatus, errorThrown) {
								if (errorThrown && errorThrown.trim() != '') {
									errorThrown = errorThrown + ': ' + qXHR.responseText;
								} else {
									errorThrown = qXHR.responseText;
								}
								serverError(gadget, errorThrown);
								gadgets.window.adjustHeight();
								/*
								var errors = {
									errors: {
										error: errorThrown
									}
								};
								qXHR.status = 500;
								qXHR.responseText = JSON.stringify(errors);
								*/
							}
			    		}
			    	}
    			}]
		   	}
		});
	}
})(AJS.$);
