(function ($) {

    RAVEN.namespace("RAVEN.Gadget");

    function message(title, message, type) {
        clearMessage();
        if (!type) {
            type = 'error';
        }
        AJS.messages[type]($('#gadget-message'), {
            title: title,
            body: message
        });
    }

    function serverError(gadget, message) {
        var obj = JSON.parse(message);

        var msg = (obj && obj.message) ? msg = obj.message : msg = gadget.getMsg('gadget.tests-evolution.common.error.data');

        $('.gadget .view').append('<div id="gadget-message" style="margin:10px;"></div>');
        AJS.messages['error']($('#gadget-message'), {
            body: msg
        });
    }

    function clearMessage() {
        $('#ag-sys-msg').empty();
        $('#gadget-message').empty();
    }

    function createTestsEvolutionChart(gadget, data) {
        var content = $('<div id="gadget-content"><div id="gadget-message"></div></div>');
        gadget.getView().html(content);

        if (data && data.labels.length > 0) {
            content = RAVEN.Gadget.renderTestsEvolution({'view': gadget.getPref("view")});

            gadget.getView().html(content);
            var chart = $("#tests-evolution-chart")[0].getContext("2d");
            window.chart = new Chart(chart, {
                type: 'line',
                data: data,
                options: {
                    elements: {
                        line: {
                            tension: 0
                        }
                    },
                    scales: {
                        yAxes: [
                            {
                                display: true,
                                ticks: {
                                    beginAtZero: true,
                                    callback: function(value, index, values) {
                                        if (Math.floor(value) === value) {
                                            return value;
                                        }
                                    }
                                },
                                gridLines: {
                                    display: true
                                },
                                stacked: false
                            }
                        ],
                        xAxes: [
                            {
                                gridLines: {
                                    display: true
                                },
                                stacked: false
                            }
                        ]
                    },
                    tooltips: {
                        mode: 'x-axis',
                        callbacks: {}
                    },
                    hover: {
                        mode: 'x-axis'
                    }
                }
            });

            // Render table skeleton
            var tableContent = RAVEN.Gadget.renderTestsEvolutionTable({'data': data});

            $('#tests-evolution-table').html(tableContent);
            $('#tests-evolution-table').xrayDataTable({
                "sPaginationType": "full_numbers",
                "bSort": false,
                "aaSorting": [],
                "bLengthChange": false,
                "bFilter": false,
                "iDisplayLength": parseInt(gadget.getPref("numberResults"))
            });
            $('#tests-evolution-table').on('draw', function () {
                gadgets.window.adjustHeight();
            });

            if (gadget.getPref("view") === "chart") {
                $('#tests-evolution-table_wrapper').hide();
            } else {
                $('#tests-evolution-chart').hide();
            }
        } else if (data.hasError) {
            message(null, gadget.getMsg("gadget.tests-evolution.common.test-exec-exceeded"), 'warning');
        } else {
            message(null, gadget.getMsg("gadget.tests-evolution.common.empty"), 'info');
        }
    }

    function setTitle(gadget) {
        var useCustomTitle = gadget.getPref("useCustomTitle");
        var customTitle = gadget.getPref("customTitle");
        var testFilterId = gadgets.util.unescapeString(gadget.getPref("testFilterId"));
        var testExecFilterId = gadgets.util.unescapeString(gadget.getPref("testExecFilterId"));
        var dateScope = gadget.getPref("dateScope");
        var startDate = gadget.getPref("startDate");
        var endDate = gadget.getPref("endDate");
        var timeFrame = gadget.getPref("timeFrame");
        var testRunAssignee = gadgets.util.unescapeString(gadget.getPref("testRunAssignee"));
        var testRunStatus = gadget.getPref("testRunStatus");

        var params = "";

        if (useCustomTitle) {
            params = params + '&useCustomTitle=' + useCustomTitle;
        }

        if (customTitle && customTitle !== '') {
            params = params + '&customTitle=' + customTitle;
        }

        if (dateScope) {
            params = params + '&dateScope=' + dateScope;
        }

        if (startDate) {
            params = params + '&startDate=' + startDate;
        }

        if (endDate) {
            params = params + '&endDate=' + endDate;
        }

        if (timeFrame) {
            params = params + '&timeFrame=' + timeFrame;
        }

        if (testRunAssignee && testRunAssignee !== '__NONE__') {
            params = params + '&testRunAssignee=' + testRunAssignee;
        }

        if (testRunStatus && testRunStatus !== '__ALL__') {
            params = params + '&testRunStatus=' + testRunStatus;
        }

        if (testFilterId) {
            // remove prefix "filter-"
            params = params + "&testFilterId=" + testFilterId.substring(7);
        }

        if (testExecFilterId) {
            // remove prefix "filter-"
            params = params + "&testExecFilterId=" + testExecFilterId.substring(7);
        }

        $.ajax({
            type: "GET",
            url: gadget.getBaseUrl() + '/rest/raven/1.0/chart/test/evolution/title?' + params,
            contentType: "application/json",
            success: function (data) {
                if (data && data.titleParams) {
                    var title = AJS.format(gadget.getMsg("gadget.tests-evolution.title"), data.titleParams);
                    gadgets.window.setTitle(title);
                } else {
                    var title = gadget.getMsg("gadget.tests-evolution.title.default");
                    gadgets.window.setTitle(title);
                }
            }
        });
    }

    RAVEN.Gadget.testsEvolution = function (contextPath) {
        return AJS.Gadget({
            baseUrl: contextPath, /* Used to make base url available to gadget */
            useOauth: "/rest/gadget/1.0/currentUser",
            config: {
                descriptor: function (args) {
                    var gadget = this;

                    $(document).off('click.raven');
                    $(document).off('change.raven');

                    $(document).on('click.raven.view', '#button-view-trigger', function () {
                        if ($('#tests-evolution-table').is(":visible")) {
                            $('#tests-evolution-table_wrapper').hide();
                            $('#tests-evolution-chart').show();
                            $('#button-view-trigger').text(gadget.getMsg("gadget.common.show-as-table"));
                            gadgets.window.adjustHeight();
                            gadget.savePref("view", "chart");
                        } else {
                            $('#tests-evolution-chart').hide();
                            $('#tests-evolution-table_wrapper').show();
                            $('#button-view-trigger').text(gadget.getMsg("gadget.common.show-as-chart"));
                            gadgets.window.adjustHeight();
                            gadget.savePref("view", "table");
                        }
                    });

                    $(document).on('change.raven.title', '#use-custom-title', function () {
                        var useCustomTitle = $("#use-custom-title").attr('checked');
                        if (useCustomTitle) {
                            $("#customTitle").prop('disabled', false);
                        } else {
                            $("#customTitle").prop('disabled', true);
                        }
                    });

                    var customTitleDisabled = true;
                    var useCustomTitle = gadget.getPref("useCustomTitle");
                    if (useCustomTitle === "true") {
                        customTitleDisabled = false;
                    }

                    // register event handler for toggling date range fields
                    $(document).on('change.raven.dateScope', '#dateScope', function () {
                        var selectedDateScope = $("#dateScope").find("option:selected").val();
                        if (selectedDateScope === "custom") {
                            $("#date-pickerend").prop('disabled', false);
                            $("#date-pickerstart").prop('disabled', false);
                            $("#timeFrame").prop('disabled', true);
                        } else if (selectedDateScope === "timeFrame") {
                            $("#date-pickerend").prop('disabled', true);
                            $("#date-pickerstart").prop('disabled', true);
                            $("#timeFrame").prop('disabled', false);
                        } else {
                            $("#date-pickerend").prop('disabled', true);
                            $("#date-pickerstart").prop('disabled', true);
                            $("#timeFrame").prop('disabled', true);
                        }
                    });

                    var customDateDisabled = true;
                    var timeFrameDisabled = true;
                    var selectedDateScope = gadget.getPref("dateScope");
                    if (selectedDateScope === "custom") {
                        customDateDisabled = false;
                    } else if (selectedDateScope === "timeFrame") {
                        timeFrameDisabled = false;
                    }

                    var statuses = $.map(args.statuses, function (p) {
                        return {
                            label: p,
                            value: p
                        };
                    });

                    statuses = [{
                        label: "All",
                        value: "__ALL__"
                    }].concat(statuses);

                    var filters = null;
                    $.each(args.filters.options, function () {
                        if (this.group.label === "Filters") {
                            filters = this.group;
                        }
                    });

                    if (filters) {
                        var testFilterId = gadget.getPref("testFilterId");
                        if (testFilterId) {
                            var testFilter = _.find(filters.options, function (f) {
                                return f.value === testFilterId;
                            });
                            if (testFilter) {
                                gadget.testFilterName = testFilter.label;
                            }
                        } else {
                            gadget.testFilterName = null;
                        }

                        var testExecFilterId = gadget.getPref("testExecFilterId");
                        if (testExecFilterId) {
                            var testExecFilter = _.find(filters.options, function (f) {
                                return f.value === testExecFilterId;
                            });
                            if (testExecFilter) {
                                gadget.testExecFilterName = testExecFilter.label;
                            }
                        } else {
                            gadget.testExecFilterName = null;
                        }
                    } else {
                        gadget.testFilterName = null;
                        gadget.testExecFilterName = null;
                    }

                    var testSearchParam;
                    if (/^jql-/.test(this.getPref("testFilterId")) || this.getPref("isPopup") === "true") {
                        testSearchParam =
                            {
                                userpref: "testFilterId",
                                type: "hidden",
                                value: gadgets.util.unescapeString(gadget.getPref("testFilterId"))
                            };
                    }
                    else {
                        testSearchParam = RAVEN.gadget.fields.filterPicker2(
                            gadget,
                            "testFilterId",
                            gadget.testFilterName,
                            gadget.getMsg("gadget.common.test-filter.label")
                        );
                    }

                    var testExecSearchParam;
                    if (/^jql-/.test(this.getPref("testExecFilterId")) || this.getPref("isPopup") === "true") {
                        testExecSearchParam =
                            {
                                userpref: "testExecFilterId",
                                type: "hidden",
                                value: gadgets.util.unescapeString(gadget.getPref("testExecFilterId"))
                            };
                    }
                    else {
                        testExecSearchParam = RAVEN.gadget.fields.filterPicker2(
                            gadget,
                            "testExecFilterId",
                            gadget.testExecFilterName,
                            gadget.getMsg("gadget.common.testexec-filter.label")
                        );
                    }

                    gadget.startDateField = RAVEN.gadget.fields.simpleDatePicker(gadget, 'start', 'startDate', gadget.getMsg("gadget.common.date.start"), '', customDateDisabled);
                    gadget.endDateField = RAVEN.gadget.fields.simpleDatePicker(gadget, 'end', 'endDate', gadget.getMsg("gadget.common.date.end"), '', customDateDisabled);
                    gadget.timeFrameField = RAVEN.Gadget.fields.timeFrameSelect(gadget, "timeFrame", gadget.getMsg("gadget.tests-evolution.common.field.time-frame"), "", timeFrameDisabled);

                    gadget.assigneeField = RAVEN.gadget.fields.assigneePicker(gadget, "testRunAssignee", gadget.getMsg("gadget.common.field.assignee.label") + ":", gadget.getMsg("gadget.common.field.assignee.desc"));

                    return {
                        action: '/rest/raven/1.0/chart/test/evolution/validate',
                        theme: function () {
                            if (gadgets.window.getViewportDimensions().width < 450) {
                                return "gdt top-label";
                            } else {
                                return "gdt";
                            }
                        }(),
                        fields: [
                            {
                                id: "customTitle",
                                userpref: "customTitle",
                                label: gadget.getMsg("gadget.common.field.custom-title.label") + ":",
                                type: "custom",
                                value: gadget.getPref("customTitle"),
                                template: function () {
                                    var disabled = "";
                                    if (customTitleDisabled === true) {
                                        disabled = " disabled"
                                    }
                                    return '<input type="text" class="text" value="' + gadget.getPref("customTitle") + '" id="customTitle" name="customTitle"' + disabled + '>';
                                }
                            }, {
                                userpref: "useCustomTitle",
                                type: "checkbox",
                                options: [
                                    {
                                        id: "use-custom-title",
                                        label: gadget.getMsg("gadget.common.field.custom-title.desc"),
                                        value: "true",
                                        selected: gadget.getPref("useCustomTitle") === 'true'
                                    }
                                ]
                            },
                            testSearchParam,
                            testExecSearchParam,
                            {
                                id: "dateScope",
                                userpref: "dateScope",
                                label: gadget.getMsg("gadget.tests-evolution.common.field.date-range.label"),
                                description: "",
                                type: "select",
                                selected: gadget.getPref("dateScope"),
                                options: [
                                    {
                                        label: gadget.getMsg("gadget.tests-evolution.common.field.date-range.custom"),
                                        value: "custom"
                                    },
                                    {
                                        label: gadget.getMsg("gadget.tests-evolution.common.field.time-frame"),
                                        value: "timeFrame"
                                    },
                                    {
                                        label: gadget.getMsg("gadget.tests-evolution.common.field.implicit"),
                                        value: "implicit"
                                    }
                                ]
                            },
                            gadget.startDateField,
                            gadget.endDateField,
                            gadget.timeFrameField,
                            gadget.assigneeField,
                            {
                                userpref: "testRunStatus",
                                label: gadget.getMsg("gadget.common.field.status.label") + ":",
                                description: gadget.getMsg("gadget.common.field.status.desc"),
                                type: "select",
                                selected: gadget.getPref("testRunStatus"),
                                options: statuses
                            },
                            {
                                userpref: "numberResults",
                                label: gadget.getMsg("gadget.common.field.number-of-results.label") + ":",
                                description: gadget.getMsg("gadget.common.field.number-of-results.desc"),
                                type: "text",
                                value: gadget.getPref("numberResults")
                            },
                            AJS.gadget.fields.nowConfigured()
                        ]
                    };
                },
                args: [
                    {
                        key: "statuses",
                        ajaxOptions: {
                            url: "/rest/raven/1.0/chart/test/statuses",
                            contentType: "application/json"
                        }
                    },
                    {
                        key: "filters",
                        ajaxOptions: {
                            url: "/rest/gadget/1.0/filtersAndProjects",
                            contentType: "application/json"
                        }
                    }
                ]
            },
            view: {
                enableReload: true,
                onResizeReload: true,
                onResizeAdjustHeight: false,
                template: function (args) {
                    var gadget = this;

                    // set gadget title
                    setTitle(this);

                    // server errors and timeout errors are not catched by the error handler of the REST date request !!!
                    if (!args.data) {
                        message(null, gadget.getMsg("gadget.tests-evolution.common.error.data"));
                    } else {
                        clearMessage();
                        createTestsEvolutionChart(gadget, args.data);

                        $(document).off('click.raven');

                        $(document).on('click.raven.view', '#button-view-trigger', function () {
                            if ($('#tests-evolution-table').is(":visible")) {
                                $('#tests-evolution-table_wrapper').hide();
                                $('#tests-evolution-chart').show();
                                $('#button-view-trigger').text(gadget.getMsg("gadget.common.show-as-table"));
                                gadget.savePref("view", "chart");
                            } else {
                                $('#tests-evolution-chart').hide();
                                $('#tests-evolution-table_wrapper').show();
                                $('#button-view-trigger').text(gadget.getMsg("gadget.common.show-as-chart"));
                                gadget.savePref("view", "table");
                            }
                            gadgets.window.adjustHeight();
                        });
                    }
                },
                args: [{
                    key: "data",
                    ajaxOptions: function () {
                        var gadget = this;

                        var dateScope = this.getPref("dateScope");
                        var startDate = this.getPref("startDate");
                        var endDate = this.getPref("endDate");
                        var timeFrame = this.getPref("timeFrame");
                        var testRunAssignee = gadgets.util.unescapeString(this.getPref("testRunAssignee"));
                        var testRunStatus = this.getPref("testRunStatus");
                        var testFilterId = gadgets.util.unescapeString(this.getPref("testFilterId"));
                        var testExecFilterId = gadgets.util.unescapeString(this.getPref("testExecFilterId"));

                        var params = '';

                        if (dateScope) {
                            params = params + 'dateScope=' + dateScope;
                        }

                        if (startDate) {
                            params = params + '&startDate=' + startDate;
                        }

                        if (endDate) {
                            params = params + '&endDate=' + endDate;
                        }

                        if (timeFrame) {
                            params = params + '&timeFrame=' + timeFrame;
                        }

                        if (testRunAssignee && testRunAssignee !== '__NONE__') {
                            params = params + '&testRunAssignee=' + testRunAssignee;
                        }

                        if (testRunStatus && testRunStatus !== '__ALL__') {
                            params = params + '&testRunStatus=' + testRunStatus;
                        }

                        if (testFilterId) {
                            // remove prefix "filter-"aaaaaaaaaa
                            params = params + "&testSavedFilter=" + testFilterId.substring(7);
                        }

                        if (testExecFilterId) {
                            // remove prefix "filter-"aaaaaaaaaa
                            params = params + "&testExecSavedFilter=" + testExecFilterId.substring(7);
                        }

                        return {
                            type: "GET",
                            url: contextPath + '/rest/raven/1.0/chart/test/evolution?' + params,
                            processData: false,
                            async: true,
                            timeout: 960000,
                            contentType: "application/json",
                            error: function (qXHR, textStatus, errorThrown) {
                                if (errorThrown && errorThrown.trim() != '') {
                                    errorThrown = errorThrown + ': ' + qXHR.responseText;
                                } else {
                                    errorThrown = qXHR.responseText;
                                }
                                serverError(gadget, errorThrown);
                                gadgets.window.adjustHeight();
                            }
                        }
                    }
                }]
            }
        });
    }
})(AJS.$);
