(function($) {

	var canEditPreCond = false;
	var inlineEdit = null;

	var preconditionTypeField = null;
	var preconditionTextField = null;
	var preconditionCucumberField = null;

    var selectedPreConditionType = null;

	function getPreConditionIssueKey() {
		return $('#raven-precondition-details-issue-key').val();
	}

    function isManualTestTypeOption(value) {
        var manualOptions = $("#manual-options").val();
        if (manualOptions) {
            var ids = JSON.parse(manualOptions);
            return _.contains(ids, parseInt(value, 10));
        }

        return false;
    }

    function isCucumberTestTypeOption(value) {
        var cucumberOptions = $("#cucumber-options").val();
        if (cucumberOptions) {
            var ids = JSON.parse(cucumberOptions);
            return _.contains(ids, parseInt(value, 10));
        }

        return false;
    }

    function isOtherTestTypeOption(value) {
        return isCucumberTestTypeOption(value) == false && isManualTestTypeOption(value) == false;
    }

    function compareTypeOptions(value1, value2) {
        return (isManualTestTypeOption(value1) === true && isManualTestTypeOption(value2) === true)
            || (isCucumberTestTypeOption(value1) === true && isCucumberTestTypeOption(value2) === true)
            || (isOtherTestTypeOption(value1) === true && isOtherTestTypeOption(value2) === true);
    }

	function updateHistory() {
		if ($('#changehistory-tabpanel').attr('class') == 'active') {
			var alt_token = $('input[name="atl_token"]').val();
			$.ajax({
				type : "GET",
				url : contextPath
					+ '/browse/'
					+ getPreConditionIssueKey()
					+ "?page=com.atlassian.jira.plugin.system.issuetabpanels:changehistory-tabpanel&_pjax=true&_="
					+ alt_token,
				contentType : "text/plain",
				success : function(data, status) {
					var tab = $(data).find('#activitymodule div.mod-content #issue_actions_container');
					var jiraTab = $('#activitymodule div.mod-content').find('#issue_actions_container');
					$(jiraTab).html(tab);
				}
			});
		}
	}

	function updateDates() {

		var updatedDate = null;

		if(AJS.$('#datesmodule #updated-date').length === 1) {
			updatedDate = AJS.$('#datesmodule #updated-date time.livestamp');
		} else if(AJS.$('#datesmodule #updated-val').length === 1) {
			updatedDate = AJS.$('#datesmodule #updated-val time.livestamp');
		} else {
			return null;
		}

		$.ajax({
			type : "GET",
			url : AJS.contextPath() + '/rest/raven/1.0/date/' + JIRA.Issue.getIssueKey() + "/updated",
			contentType : "application/json",
			success : function (data) {
				var title = data.title;
				var dateTimeIso8601 = data.dateTimeIso8601;
				updatedDate.attr("datetime", dateTimeIso8601);
				$.each(AJS.$('#datesmodule dd.date'), function() {
                    if($(this).find('#updated-date time.livestamp').length > 0 || $(this).find('#updated-val time.livestamp').length > 0) {
                        $(this).attr("title", title);
                    }
				});
				updatedDate.livestamp();
			}
		});
	}

	function toogleViewMode(sync) {
		if (isManualTestTypeOption(selectedPreConditionType) === true || isOtherTestTypeOption(selectedPreConditionType) === true) {
			// update value
			if (sync && $('.cucumber-precondition-details').is(":visible")) {
                preconditionTextField.setValue(preconditionCucumberField.getValue());
			}
			$('.text-precondition-details').show();
			$('.cucumber-precondition-details').hide();
		} else {
			// update value
			if (sync && $('.text-precondition-details').is(":visible")) {
                var textValue = preconditionTextField.getValue();
				$('#precondition-condition-hidden-value').val(textValue.raw);
				preconditionCucumberField.setValue(textValue.raw);
			}
			$('.text-precondition-details').hide();
			$('.cucumber-precondition-details').show();
		}
	}

	function onValidateSuccess(data, saveCallback, cancelCallback) {
		var conflicts = (data.conflicts != undefined && data.conflicts != -1) ? data.conflicts : -1;
		if (conflicts == 0) {
			saveCallback();
		} else if (conflicts > 0) {
			var title = AJS.I18n.getText('view-issue-precondition-section.dialogs.remove.testassoc.title');
			var msg = AJS.I18n.getText('view-issue-precondition-section.dialogs.remove.testassoc.message');
			var issues = data.conflicts_list;
			confirmationRemoveTestAssoc(title, msg, true, saveCallback, cancelCallback, issues);
		}
	}

	function onValidateError(qXHR, textStatus, errorThrown) {
		var error = null;
		$('#raven-error-message').remove(); // clear previous message
		if (errorThrown && errorThrown.trim() != '') {
			errorThrown = qXHR.responseText;
            try {
                error = JSON.parse(errorThrown);
            } catch(err) {
                // not in JSON
            }
		}

        if (error) {
            AJS.messages.error({
                title : AJS.I18n.getText('view-issue-precondition-section.save.error', error.errors),
                body : error.errorMessages,
                id : "raven-error-message"
            });
		} else {
            AJS.messages.error({
                title: "Internal Error",
                body : errorThrown,
                id : "raven-error-message"
            });
        }
	}

	function validateAssociateTests(selectValue, saveCallback, cancelCallback) {
		$.ajax({
			type : "GET",
			url : contextPath + '/rest/raven/1.0/precondition/'
				+ getPreConditionIssueKey() + '/type/'
				+ encodeURIComponent(selectValue),
			async : true,
			contentType : "application/json",
			success : function(data) { 
				if (data && data != undefined) {
					onValidateSuccess(data, saveCallback, cancelCallback);
				} else {
					cancelCallback();
				}
			},
			error : function(qXHR, textStatus, errorThrown) {
				cancelCallback();
				onValidateError(qXHR, textStatus, errorThrown);
			}
		});
	}

	function confirmationRemoveTestAssoc(title, message, reset, saveCallback, cancelCallback, dataList) {
		var dialog = new JIRA.FormDialog({
			id: 'raven-change-precondition-type',
			content: function (callback) {
        		var innerHtml = RAVEN.Dialog.confirmationDialogWithData({title:title, message:message, data:dataList, url:contextPath});
				callback(innerHtml);
			},
			submitHandler: function (e) {
				saveCallback();
        		dialog.hide();
				e.preventDefault();
			},
			autoClose : true
		});
		dialog.bind('Dialog.hide', function(e, popup, hideReason) { 
			if (hideReason === JIRA.Dialog.HIDE_REASON.cancel || hideReason === JIRA.Dialog.HIDE_REASON.escape) {
				cancelCallback();
			}
		});
		
		dialog.show();
	}
	
	function initialize() {
		AJS.messages.setup();

		initView();
	}

	function initView() {
		canEditPreCond = $('#can-edit-precondition').val() === 'true';

        selectedPreConditionType = $('#precondition-type-field-value').val();

		// set view mode: text or cucumber tests
		toogleViewMode(false);
		
		// inline edit support
		initInlineEditFields(canEditPreCond);
	}

	function fieldSaveError(field, error) {
		$('#raven-precondition-details-message-bar').empty(); // clear previous message
		
		var fieldName = $(field.el).data('field-name');
		AJS.messages.error("#raven-precondition-details-message-bar", {
			title : AJS.I18n.getText('view-issue-precondition-section.save.error', fieldName),
			body : error
		});
	}
	
	function initInlineEditFields(canEditTestExec) {
		// create Inline Edit
		preconditionTypeField = new RAVEN.InlineEdit.SelectPickerField(
			$('#precondition-type-field'),
			{
				validate: function(saveCallback, cancelCallback) {
					var newSelectedValue = $('#precondition-type-field-select').val();
					validateAssociateTests(newSelectedValue, saveCallback, cancelCallback);
				},
				success: function() {
					var newSelectedValue = $('#precondition-type-field-select').val();
                    selectedPreConditionType = newSelectedValue;
					toogleViewMode(true);
					updateHistory();
					updateDates();
					JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);
					$('#raven-precondition-details-message-bar').empty();
				},
				error: fieldSaveError
			}
		);
		preconditionTextField = new RAVEN.InlineEdit.WikiTextField(
			$('#precondition-text-field'),
			{
				success: function() {
					updateHistory();
					updateDates();
					$('#raven-precondition-details-message-bar').empty();
				},
				error: fieldSaveError
			}
		);

		preconditionCucumberField = new RAVEN.InlineEdit.AceEditorField(
			$('#precondition-cucumber-field'),
			{
				success: function() {
					updateHistory();
					updateDates();
					$('#raven-precondition-details-message-bar').empty();
				},
				error: fieldSaveError
			}
		);

		inlineEdit = new RAVEN.InlineEdit();
	}

	// register for NEW_CONTENT_ADDED
	JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, function(e, context, reason) {
		// FIXME: Both load page and refresh are called when loading a page and the table is loaded 2 times
		// test if our view is already loaded because pageLoaded event is called 2 times in the issue search page
		if (reason == JIRA.CONTENT_ADDED_REASON.pageLoad || reason == JIRA.CONTENT_ADDED_REASON.panelRefreshed) {
            var $section = $('#raven-precondition-details');
            if ($section.length > 0 && !$section.attr('data-initialized')) {
                initialize();
				$('#raven-precondition-details').attr("data-initialized", "true");
            }
        }
	});

})(AJS.$);
