(function($) {

    RAVEN.namespace("RAVEN.Project");

    // HACK: older versions of JIRA (6.0) do not contain _.findWhere
    if (_ && !_.findWhere) {
        _.findWhere = function(list, properties) {
            return _.first(_.where(list, properties));
        }
    }

    /**
     * This Model defines the column layout for the given user
     */
    RAVEN.Project.TestsColumnLayoutModel = Backbone.Model.extend({
        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/testexec/columns/config';
        },

        defaults: function() {
            return {
                visibleColumnLayoutItems: [],
                availableColumnLayoutItems: []
            }
        }

    });

    /**
     * This is the filters view for the Test Runs table on a Test issue
     */
    RAVEN.Project.TestExecsColumnLayoutView = Backbone.View.extend({
        el: '.columns-config-button-wrapper',
        model: new RAVEN.Project.TestsColumnLayoutModel(),

        /**
         * Parent Tests view
         */
        mainView: null,
        
        refreshFunction: null,

        /**
         * The inline dialog object for configuring columns
         */
        columnsDialog: null,
        
        keyFilterAllColumnLayoutItems: [],
        
        keyFilterAllVisibleColumnsLayoutItems: [],
        
        originalAvailableColumnLayoutItems: null,
        
        originalVisibleColumnLayoutItems: null,
        
        _hasToRevert: false,
        

        /**
         * Unscoped Events - these events must be registered elsewhere as they will not be
         * fired by backbone because they are not within the view element. Refer to "_delegateUnscopedEvents".
         */
        unscopedEvents: {
            'click.RAVEN.Project.columns #inline-dialog-test-columns-config-dialog': '_onDialogClick',
            'submit.RAVEN.Project.columns #tests-column-configuration-form': '_onSubmitColumnConfigForm',
            'click.RAVEN.Project.columns #inline-dialog-test-columns-config-dialog .close-dialog': '_onCloseDialogClick',
            'click.RAVEN.Project.columns #tests-column-configuration-form .restore-defaults': '_onRestoreButtonClick',
            'change.RAVEN.Project.columns #tests-column-configuration-form .check-list-item input': '_onColumnChange',
            'keyup.RAVEN.Project.columns #tests-column-configuration-form .check-list-field-container input': '_onSearchInputKeyUp'
        },

        initialize: function(options) {
            var self = this;

            if (options) {
                _.extend(this, _.pick(options, "mainView"));
                _.extend(this, _.pick(options, "refreshFunction"));
            }

            this._delegateUnscopedEvents();

            self.$el.block();
            this.model.fetch({
                success: function() {
                    if (self.originalAvailableColumnLayoutItems == null ) {
                        self.originalAvailableColumnLayoutItems = self.model.get('availableColumnLayoutItems').slice();
                    }
                    if (self.originalVisibleColumnLayoutItems == null) {
                        self.originalVisibleColumnLayoutItems = self.model.get('visibleColumnLayoutItems').slice();
                    }
                    self.render();
                    self.$el.unblock();
                },
                error: function(model, response) {
                    if (self.mainView) {
                        self.mainView.message(AJS.I18n.getText('view-issue-testexec-tests-section.column-layout.view.error'));
                    }
                    self.$el.unblock();
                }
            });
        },

        render: function() {
            this.$el.empty();

            var contents = RAVEN.Project.renderColumnsConfigButton();
            this.$el.html(contents);

            this._initColumnsInlineDialog();

            this.$el.show();

            return this;
        },

        moveColumn: function(columnId, anchorColumnId) {
            var columnItems = this.model.get('visibleColumnLayoutItems');
            var columnItem = _.findWhere(columnItems, {id: columnId});
            if (columnItem) {
                var index = columnItems.indexOf(columnItem);
                if (anchorColumnId == null) {
                    this._moveToPosition(columnItems, index, 0);
                } else {
                    var anchorColumnItem = _.findWhere(columnItems, {id: anchorColumnId});
                    if (anchorColumnItem) {
                        var targetIndex = columnItems.indexOf(anchorColumnItem);
                        if (index > targetIndex) {
                            this._moveToPosition(columnItems, index, targetIndex + 1);
                        } else {
                            this._moveToPosition(columnItems, index, targetIndex);
                        }
                    }
                }

                this.model.set('visibleColumnLayoutItems', columnItems);
                this._saveModel();
            }
        },

        _moveToPosition: function(array, from, to) {
            array.splice(to, 0, array.splice(from, 1)[0]);
        },

        _initColumnsInlineDialog: function() {
            var self = this;
            
            this.keyFilterAllColumnLayoutItems = this.model.get('availableColumnLayoutItems');
            this.keyFilterAllVisibleColumnsLayoutItems = this.model.get('visibleColumnLayoutItems');
            
            this.columnsDialog = AJS.InlineDialog(AJS.$("#test-columns-config-trigger"), "test-columns-config-dialog", function($content, trigger, showPopup) {
                var inlineColumnsDialogContents = RAVEN.Project.createColumnsInlineDialog({ 'columnLayout': self.model.attributes });
                $content.html(inlineColumnsDialogContents);

                $content.find(".check-list-item").hover(function () {
                    $(this).toggleClass("active");
                });

                self.model.set('availableColumnLayoutItems', self.keyFilterAllColumnLayoutItems);
                self.model.set('visibleColumnLayoutItems', self.keyFilterAllVisibleColumnsLayoutItems);
                
                showPopup();

                return false;
            }, {
                hideDelay: null,
                addActiveClass: true,
                hideCallback: function() { 
                	self._revertColumnsToOriginal(); 
                }
            });
        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        _delegateUnscopedEvents: function() {
            var self = this;
            $(document).off('.RAVEN.Project.columns');

            $.each(this.unscopedEvents, function(key, val) {
                var eventName = key.substring(0, key.indexOf(' '));
                var eventTrigger = key.substring(key.indexOf(' ') + 1);

                $(document).on(eventName, eventTrigger, $.proxy(self[val], self));
            });
        },

        /**
         * HACK: https://jira.atlassian.com/browse/JRA-24215
         */
        _onDialogClick: function(e) {
            e.stopPropagation();
        },

        _onSubmitColumnConfigForm: function(e) {
            var self = this;

            // Stops the form submission
            e.preventDefault();

            this._saveModel();

            this._hasToRevertSelectedColumns(false);
            this.columnsDialog.remove();
        },

        _saveModel: function() {
            var self = this;

            // save model and trigger a page refresh event
            $('#raven-test').block();
            var promise = this.model.save();
            promise.done(function(data, status) {
                // trigger a refresh on the page
                // this takes too long and the user might think something went wrong
                // JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);

                // save new original
                self.originalAvailableColumnLayoutItems = self.model.get('availableColumnLayoutItems').slice();
                self.originalVisibleColumnLayoutItems = self.model.get('visibleColumnLayoutItems').slice();


                // refreshing the view direcly
                self.mainView.refresh();
            	//self.refreshFunction();

                $('#raven-testexec').unblock();
            });
            promise.fail(function() {
                if (self.mainView) {
                    self.mainView.message(AJS.I18n.getText('view-issue-testexec-tests-section.column-layout.save.error'));
                }
                $('#raven-testexec').unblock();
            });
        },
        
        _onSearchInputKeyUp: function (event) {

            var cursorposition = $("#user-column-sparkler-input")[0].selectionStart;
            var instance = $(event.target);
            if (event.keyCode === 13) {
                event.preventDefault();
                return false;
            }
            event.preventDefault();
            event.stopPropagation();
            
            var arrayModel = [];
            var arrayVisibleModel = [];
            var value = instance.val();
            
            //Filter all available columns
            //reset columns before filtering
            this.model.set('availableColumnLayoutItems', this.keyFilterAllColumnLayoutItems);
            
            this.model.get('availableColumnLayoutItems').forEach(function (currentValue) {
                if (currentValue.label.toLowerCase().indexOf(value.toLowerCase()) > -1) {
                    arrayModel.push(currentValue);
                }
            });
            //set filtered columns as available columns                        
            this.model.set('availableColumnLayoutItems', arrayModel);
            
            //Filter all visible columns
            //reset columns before filtering
            this.model.set('visibleColumnLayoutItems', this.keyFilterAllVisibleColumnsLayoutItems);
            
            this.model.get('visibleColumnLayoutItems').forEach(function (currentValue) {
                if (currentValue.label.toLowerCase().indexOf(value.toLowerCase()) > -1) {
                	arrayVisibleModel.push(currentValue);
                }
            });
            
            //set filtered columns as visible columns 
            this.model.set('visibleColumnLayoutItems', arrayVisibleModel);
            
            this.columnsDialog.show();
            this.columnsDialog.refresh();
            this.$el.unblock();
            instance.value = value;
            
            $("input#user-column-sparkler-input.aui-field.check-list-field").focus();
            $("input#user-column-sparkler-input.aui-field.check-list-field").val(value);
            $("input#user-column-sparkler-input.aui-field.check-list-field").value = value;
            $("input#user-column-sparkler-input.aui-field.check-list-field")[0].setSelectionRange(cursorposition,cursorposition);
            
            if(value.length > 0){
            	$("ul.aui-list-section.aui-last").css("border-top", "none");
            }
            
            return false;
        },
        _onColumnChange: function(e) {
        	this._hasToRevertSelectedColumns(true);
            var columnId = $(e.target).val();

            if ($(e.target).is(':checked')) {
                var columnItem = _.findWhere(this.model.get('availableColumnLayoutItems'), {id: columnId});
                if (columnItem) {
                    this.model.set('availableColumnLayoutItems', _.without(this.model.get('availableColumnLayoutItems'), columnItem));

                    this.model.get('visibleColumnLayoutItems').push(columnItem);
                }
            } else {
                var columnItem = _.findWhere(this.model.get('visibleColumnLayoutItems'), {id: columnId});
                if (columnItem) {
                    this.model.set('visibleColumnLayoutItems', _.without(this.model.get('visibleColumnLayoutItems'), columnItem));

                    this.model.get('availableColumnLayoutItems').push(columnItem);
                }
            }
        },

        _onCloseDialogClick: function(e) {
            e.preventDefault();
            e.stopPropagation();

            //Cancel all unconfirmed changes
//            if(e && e.target && e.target.className.indexOf("submit") == -1){
//            	this._revertColumnsToOriginal();
//            }
            
            this.columnsDialog.hide();

            return false;
        },
        
        _hasToRevertSelectedColumns : function (hasToRevert){
        	console.log('cid: ' + this.cid + ' hasToRevert: ' + this._hasToRevert);
        	
        	if(hasToRevert != null){
        		this._hasToRevert = hasToRevert;
        	}
        	
        	if(this._hasToRevert){
        		return true;
        	}
        	
        	return false;
        },
        
        _revertColumnsToOriginal: function(){
        	if(this._hasToRevertSelectedColumns()){
	            this.model.set('visibleColumnLayoutItems', this.originalVisibleColumnLayoutItems.slice());
	            this.model.set('availableColumnLayoutItems', this.originalAvailableColumnLayoutItems.slice());
	            
	            this.keyFilterAllColumnLayoutItems = this.model.get('availableColumnLayoutItems').slice();
	            this.keyFilterAllVisibleColumnsLayoutItems = this.model.get('visibleColumnLayoutItems').slice();
	            
	            this._hasToRevertSelectedColumns(false);
        	}
        },
        
        _onRestoreButtonClick: function(e) {
            e.preventDefault();
            e.stopPropagation();

            var self = this;

            var configuredItems = this.model.get('visibleColumnLayoutItems');
            if (configuredItems) {
                this.model.set('visibleColumnLayoutItems', []);

                _.each(configuredItems, function(columnItem) {
                    self.model.get('availableColumnLayoutItems').push(columnItem);
                });
            }

            this._saveModel();

            this.columnsDialog.remove();

            return false;
        }
    });
})(AJS.$);