(function($) {

    RAVEN.namespace("RAVEN.TestPlan");

    function getTestPlanKey() {
        return $('#raven-testplan-tests-issue-key').val();
    }

    /**
    * This Model defines the column layout for the given user
    */
    RAVEN.TestPlan.TestFiltersModel = Backbone.Model.extend({
        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/testplan/' + getTestPlanKey() + '/filters';
        },

        defaults: function() {
            return {
                statuses: []
            }
        },
    });

    /**
     * This is the filters view for the Tests table on a Test Plan issue
     */
    RAVEN.TestPlan.TestFiltersView = Backbone.View.extend({
        el: '#testplan-tests-filter',
        model: new RAVEN.TestPlan.TestFiltersModel(),

        /**
         * Parent Tests view
         */
        mainView: null,

        /**
         * Whether using Test hierarchy or not
         */
        isTestRepositoryEnabled: false,

        /**
         * The Test Plan issue key
         */
        testPlanKey: null,

        testRepositoryFolderPicker: null,

        /**
         * Test Set single select filter object
         */
        testSetSingleSelect: null,

        /**
         * Status single select filter object
         */
        statusMultiSelect: null,

        /**
         * Assignee single select filter
         */
        assigneeSingleSelect: null,

        /**
         * Stores the last selected Test Repository Folder Id
         */
        testRepositoryFolderId: null,

        /**
         * Wheither we load all descendants or only direct ones
         */
        testRepositoryFolderAllDescendants: false,

        /**
         * Stores the last selected Test Set key
         */
        selectedTestSetKey: null,

        selectedTestPath: null,

        /**
         * Stores the last selected Components
         */
        componentsMultiSelect: null,

        /**
         * Stores the last selected Test Environments
         */
        testEnvironmentsLabelPicker: null,

        /**
         * Stores the last assignee value of the filter
         */
        lastAssigneeFilterValue: null,

        /**
         * Stores the last status value of status filter
         */
        lastStatusFilterValue: null,

        /**
         * Stores the last components value of components filter
         */
        lastComponentsValue: null,

        /**
         * Stores the last test environments value of components filter
         */
        lastTestEnvironmentsValue: null,

        /**
         * Stores the last entered value on text filter
         */
        lastTextFilterValue: null,

        allDefaultItem: new AJS.ItemDescriptor({
            value: '-1',
            label: AJS.I18n.getText("common.label.all"),
            allowDuplicate: false,
            icon: null
        }),

        events: {
            'keyup #testplan-tests-filter-contains:not(.disabled)': '_onTextFilterKeyUp',
            'changeFolderPick #testplan-test-folder-filter': '_onTestRepositoryFolderFilterChanged',
            'change #testplan-tests-filter-testsets:not(.disabled)': '_onTestSetsFilterChanged',
            'change #testplan-tests-filter-assignee:not(.disabled)': '_onAssigneeFilterChanged',
            'change #testplan-tests-filter-status:not(.disabled)': '_onStatusFilterChanged',
            'change #testplan-tests-filter-component:not(.disabled)': '_onComponentsFilterChanged',
            'change #testplan-tests-filter-test-environment:not(.disabled)': '_onTestEnvironmentsFilterChanged',
            'change #testplan-all-descendants:not(.disabled)': '_onTestRepositoryFolderAllDescendantsChanged',
            'click .item-delete': '_onItemRemoved',
            'click #testplan-tests-filter-clean': '_onCleanFilterClick'
        },

        initialize: function(options) {
            var self = this;

            if (options) {
                _.extend(this, _.pick(options, "mainView"));
            }

            this.testPlanKey = getTestPlanKey();

            this.isTestRepositoryEnabled = $('#raven-test-repository-enabled').val() === "true";

            self.$el.block();
            this.model.fetch({
                success: function() {
                    self.render();
                    self.$el.unblock();
                },
                error: function(model, response) {
                    if (self.mainView) {
                        self.mainView.message(AJS.I18n.getText('view-issue-testplan-tests-section.table-filter.error'));
                    }
                    self.$el.unblock();
                }
            });
        },

        render: function() {
            this.$el.empty();

            var contents = RAVEN.TestPlan.renderTestFilters({ statuses: this.model.get('statuses'), isTestRepositoryEnabled: this.isTestRepositoryEnabled });
            this.$el.html(contents);

            this._initializeFilters();

            this.$el.show();

            return this;
        },

        _initializeFilters: function() {
            var self = this;

            if ($('#plan-entries-table').length > 0) {
                var $testRepositoryFolderPicker = $('#testplan-test-folder-filter');
                this._createTestRepositoryFolderPicker($testRepositoryFolderPicker);

                this._createTestSetPicker();
            }

            this.statusMultiSelect = new AJS.MultiSelect({
                element : self.$el.find('#testplan-tests-filter-status'),
                revertOnInvalid: true
            });

            this.assigneeSingleSelect = new AJS.SingleSelect({
                element: self.$el.find('#testplan-tests-filter-assignee'),
                submitInputVal: true,
                revertOnInvalid: true,
                showDropdownButton: true,
                errorMessage: AJS.I18n.getText("user.picker.invalid.user", "'{0}'"),
                ajaxOptions: {
                    url: contextPath + "/rest/api/2/user/picker",
                    query: true, // keep going back to the sever for each keystroke
                    data: {
                        showAvatar: true
                    },
                    formatResponse: $.proxy(self._formatAssigneeFilterResponse, self)
                },
                inputText: null
            });

            this.componentsMultiSelect = new AJS.MultiSelect({
    			element: $("#testplan-tests-filter-component"),
    			showDropdownButton: true,
    			submitInputVal: true,
    			noQueryNoRequest: false,
    			revertOnInvalid: true,
    			loadOnInit: true,
    			ajaxOptions: {
    				query: true,
    				url: contextPath + '/rest/api/2/project/' + $('#testplan-project-key').val() + '/components',
    				formatResponse: function (data) {
    					var optgroup = new AJS.GroupDescriptor({
    						weight: 0,
    						id: "all-filter-opt-group",
    						replace: true
    					});

    					$(data).each(function(i, status) {
    						this.label = this.name;
    						var item = new AJS.ItemDescriptor({
    							value: this.name,
    							label: this.name,
    							title: this.description,

    							highlighted: true,
    						});
    						optgroup.addItem(item);
    					});

    					return [optgroup];
    				}
    			}
    		});

            var getDefaultOptionLabelPicker = AJS.LabelPicker.extend({
                _getDefaultOptions: function () {
                    return jQuery.extend(true, this._super(), {
                        ajaxOptions: {
                            url: contextPath + "/rest/api/1.0/labels/suggest?customFieldId=" + $('#raven-testplan-testexec-testEnvironmentsId').val(),
                            query: true,
                            formatResponse: this._formatTestEnvironmentsResponse
                        }
                    });
                }
            });

            this.testEnvironmentsLabelPicker = new getDefaultOptionLabelPicker({
                element: $("#testplan-tests-filter-test-environment"),
                itemAttrDisplayed: "label",
                showDropdownButton: true,
                removeOnUnSelect: false,
                submitInputVal: false,
                errorMessage: AJS.I18n.getText("raven-admin-config.requirement-coverage.ignore-requirements-in-labels.error"),
                maxInlineResultsDisplayed: 15
            });

        },

        /**
         * Create Test Repository Folder Picker filter
         */
        _createTestRepositoryFolderPicker: function($picker) {
            if(this.isTestRepositoryEnabled) {
                this.testRepositoryFolderPicker = new RAVEN.TestRepository.Fields.FolderPicker({
                    $picker: $picker,
                    repositoryPath: 'testPlanRepository',
                    folderJQLName: 'testPlanFolderTests',
                    entityKey: $('#raven-testplan-tests-issue-key').val(),
                    siblingDropdownElements: this._getSiblingDropdownElements()
                });
            }
        },

        _getSiblingDropdownElements: function() {
            return [
                'testplan-tests-filter-testsets-field',
                'testplan-tests-filter-assignee-field',
                'testplan-tests-filter-status-textarea',
                'testplan-tests-filter-component-multi-select',
                'testplan-tests-filter-test-environment-multi-select'
            ];
        },

        /**
         * Create select Test Set filter
         */
        _createTestSetPicker: function() {
            var self = this;

            // prevent form submission
            this.$el.find('form.aui').submit(false);

            var $selectField = this.$el.find('#testplan-tests-filter-testsets');

            var path = contextPath + '/rest/raven/1.0/testplan/' + getTestPlanKey() + '/testset/query';

            this.testSetSingleSelect = new AJS.SingleSelect({
                element: $selectField,
                revertOnInvalid: true,
                submitInputVal: true,
                itemAttrDisplayed: "label",
                ajaxOptions: {
                    query: true,
                    processData: true,
                    data: {
                    	issueType: 'TESTSET',
                    	showSubTasks: true
                    	},
                    url: path,
                    suggestionsHandler: null,
                    formatResponse: function (response) {
                        if (response) {
                            var items = _.map(response, function(item) {
                                return new AJS.ItemDescriptor({
                                    value: item.key,
                                    label: item.key + " - " + item.summary,
                                    icon: contextPath + '/download/resources/com.xpandit.plugins.xray/images/test-set.png',
                                    html: '<img class="icon" alt="" src="' + contextPath + '/download/resources/com.xpandit.plugins.xray/images/test-set.png"><div style="overflow: hidden;text-overflow: ellipsis;" title="'+item.key + ' - ' + item.summary+'" >' +item.key + ' - ' + item.summary+'</div>',
                                    styleClass: "aui-iconised-link",
                                    allowDuplicate: false
                                });
                            });

                            return [self.allDefaultItem].concat(items);
                        }

                        return null;
                    }
                }
            });
        },

        /**
         * @Deprecated
         */
        _createTestSetOptions: function($selectField, data) {
            $selectField.append('<option value="-1" style="background-image: none;">' + AJS.I18n.getText("common.label.all") + '</option>');
            $(data).each(function(i, proj) {
                $selectField.append('<option value="' + proj.key + '">' + proj.name + '</option>');
            });
        },

        _formatAssigneeFilterResponse: function(data) {
            var ret = [];

            var suggestionsGroupDescriptor = new AJS.GroupDescriptor({
                weight: 0, // order or groups in suggestions dropdown
                label: AJS.I18n.getText("common.label.user-suggestions")
            });

            suggestionsGroupDescriptor.addItem(new AJS.ItemDescriptor({
                value: "-1",
                label: AJS.I18n.getText("common.label.all"), // title of lozenge
                icon: null,
                allowDuplicate: false,
                highlighted: true
            }));

            suggestionsGroupDescriptor.addItem(new AJS.ItemDescriptor({
                value: "\u241Eunassigned\u241E",
                label: AJS.I18n.getText("common.label.unassigned"), // title of lozenge
                icon: null,
                allowDuplicate: false,
                highlighted: true
            }));

            var currentUser = this.model.get('currentUser');
            if (currentUser) {
                suggestionsGroupDescriptor.addItem(new AJS.ItemDescriptor({
                    value: currentUser.key,
                    label: AJS.I18n.getText('common.label.current-user') + " - " + currentUser.displayName, // title of lozenge
                    icon: currentUser.avatarUrls['16x16'],
                    fieldText: currentUser.displayName,
                    allowDuplicate: false,
                    highlighted: true
                }));
            }

            ret.push(suggestionsGroupDescriptor);

            var groupDescriptor = new AJS.GroupDescriptor({
                label: data.header
            });
            $(data.users).each(function(){
                groupDescriptor.addItem(new AJS.ItemDescriptor({
                    value: this.key, // value of item added to select
                    label: this.displayName, // title of lozenge
                    html: this.html,
                    icon: this.avatarUrl,
                    allowDuplicate: false,
                    highlighted: true
                }));
            });

            ret.push(groupDescriptor);
            return ret;
        },

        _formatTestEnvironmentsResponse: function (data) {

            var optgroup = new AJS.GroupDescriptor({
                label: AJS.I18n.getText("Project Versions"),
                type: "optgroup"
            });

            if (data) {
                AJS.$.each(data, function () {
                    optgroup.addItem(new AJS.ItemDescriptor({
                        value: this.id,
                        label: this.name,
                        title: this.description,
                        highlighted: true
                    }));
                });
            }
            return [optgroup];
        },

        /**
         * Event handler for when Test Repository Folder filter option is changed
         */
        _onTestRepositoryFolderFilterChanged: function(e, folderId) {
            if (this.testRepositoryFolderId !== folderId) {
                this.testRepositoryFolderId = folderId ? folderId : '';

                this._enableTestPlanAllDescendantsOption(folderId === "all-tests" || folderId === "folder-root");

                // perform search
                this._applyFilters();
            }
        },

        /**
         * @param enable whatever is to enable or not the checkbox for all descendants
         * @private
         */
        _enableTestPlanAllDescendantsOption: function(enable) {
            if(enable){
                $('#testplan-all-descendants').prop('disabled', true);
                $('#testplan-all-descendants').prop('checked', false);
                $('#testplan-cb-description').addClass('filter disabled');
            } else {
                $('#testplan-all-descendants').prop('disabled', false);
                $('#testplan-cb-description').removeClass('disabled');
            }
        },

        /**
         * Event handler for when Test Set filter option is changed
         */
        _onTestSetsFilterChanged: function(e) {
            var $selectField = this.$el.find('#testplan-tests-filter-testsets');
            var newSelectedValue = $selectField.find('option:selected').attr('value');
            if (this.selectedTestSetKey !== newSelectedValue) {
                this.selectedTestSetKey = newSelectedValue ? newSelectedValue : '';
                // perform search
                this._applyFilters();
            }
        },

        /**
         * Event handler for when Assignee filter option is changed
         */
        _onAssigneeFilterChanged: function(e) {
            var $selectField = this.$el.find('#testplan-tests-filter-assignee');
            var newSelectedValue = $selectField.find('option:selected').attr('value');
            if (this.lastAssigneeFilterValue !== newSelectedValue) {
                this.lastAssigneeFilterValue = newSelectedValue ? newSelectedValue : '';
                if(newSelectedValue === "-1" ){
                	$("#testplan-tests-filter-assignee-single-select").removeClass("aui-ss-has-entity-icon");
                }
                // perform search
                this._applyFilters();
            }
        },

        /**
         * Event handler for when Status filter option is changed
         */
        _onStatusFilterChanged: function(e) {
            var $selectField = this.$el.find('#testplan-tests-filter-status');
            var newSelectedValue = $selectField.val();
            if (this.lastStatusFilterValue !== newSelectedValue) {
                this.lastStatusFilterValue = newSelectedValue ? newSelectedValue : '';
                // perform search
                this._applyFilters();
            }
        },

        _onComponentsFilterChanged : function(e) {
        	var $selectField = this.$el.find("#testplan-tests-filter-component");
        	var newSelectedValue = $selectField.val();
        	if (this.lastComponentsValue !== newSelectedValue ) {
                this.lastComponentsValue = newSelectedValue ? newSelectedValue : '';
                // perform search
                this._applyFilters();
        	}
        },

        _onTestEnvironmentsFilterChanged : function(e) {
            var $selectField = this.$el.find("#testplan-tests-filter-test-environment");
            var newSelectedValue = $selectField.val();
            if (this.lastTestEnvironmentsValue !== newSelectedValue ) {
                this.lastTestEnvironmentsValue = newSelectedValue ? newSelectedValue : '';
                // perform search
                this._applyFilters();
            }
        },

        _onTestRepositoryFolderAllDescendantsChanged: function(e) {
            var newSelectedValue = this.$el.find('#testplan-all-descendants:checked').length > 0;
            if (this.testRepositoryFolderAllDescendants !== newSelectedValue ) {
                this.testRepositoryFolderAllDescendants = newSelectedValue;
                // perform search
                this._applyFilters();
            }
        },

        _onItemRemoved : function(e) {
        	var newSelectedValue = $("#testplan-tests-filter-component").val();

        	if (this.lastComponentsValue !== newSelectedValue ) {
                this.lastComponentsValue = newSelectedValue ? newSelectedValue : '';
        	}

            var newSelectedStatusValue = $  ('#testplan-tests-filter-status').val();
            if (this.lastStatusFilterValue !== newSelectedStatusValue) {
                this.lastStatusFilterValue = newSelectedStatusValue ? newSelectedStatusValue : '';
            }

            var newSelectedTestEnvironmentsValue = $('#testplan-tests-filter-test-environment').val();
            if (this.lastTestEnvironmentsValue !== newSelectedTestEnvironmentsValue) {
                this.lastTestEnvironmentsValue = newSelectedTestEnvironmentsValue ? newSelectedTestEnvironmentsValue : '';
            }

            this._applyFilters();
        },

        /**
         * Event handler for when text filter is modified
         */
        _onTextFilterKeyUp: function(e) {
            var $textField = this.$el.find('#testplan-tests-filter-contains');
            if (this.lastTextFilterValue !== $textField.val()) {
                this.lastTextFilterValue = $textField.val();
                // perform search
                this._applyFilters();
            }
        },

        _applyFilters: function() {
            var data = [];

            if (this.testRepositoryFolderId) {
                data.push({ "name": "filter_test-repository-folder-id", "value": this.testRepositoryFolderId});
            }
            if (this.testRepositoryFolderAllDescendants) {
                data.push({ "name": "filter_test-repository-folder-all-descendants", "value": this.testRepositoryFolderAllDescendants});
            }
            if (this.selectedTestSetKey && this.selectedTestSetKey != '-1') {
                data.push({ "name": "filter_test_set", "value": this.selectedTestSetKey});
            }
            if (this.lastAssigneeFilterValue && this.lastAssigneeFilterValue != '-1') {
                data.push({ "name": "filter_testrun_assignee", "value": this.lastAssigneeFilterValue});
            }
            if (this.lastStatusFilterValue) {
                data.push({ "name": "filter_testrun_status", "value": this.lastStatusFilterValue});
            }
            if (this.lastComponentsValue) {
                data.push({ "name": "filter_component", "value": this.lastComponentsValue});
            }
            if (this.lastTestEnvironmentsValue) {
                data.push({ "name": "filter_test_environment", "value": this.lastTestEnvironmentsValue});
            }
            if (this.lastTextFilterValue) {
                data.push({ "name": "filter_text", "value": this.lastTextFilterValue});
            }

            this.mainView.filter(data);
        },

        /**
         * Event handler for when clean filter button is clicked
         */
        _onCleanFilterClick: function(e) {
            var $textField = this.$el.find('#testplan-tests-filter-contains');

            $('#testplan-tests-filter-contains').addClass('disabled');
            $('#testplan-tests-filter-testsets').addClass('disabled');
            $('#testplan-tests-filter-assignee').addClass('disabled');
            $("#testplan-tests-filter-component").addClass('disabled');
            $('#testplan-tests-filter-status').addClass('disabled');

            $textField.val('');
            this.testRepositoryFolderPicker.clearPicker();
            this.testSetSingleSelect.setSelection(this.allDefaultItem);
            this.assigneeSingleSelect.setSelection(this.allDefaultItem);
            this.statusMultiSelect.clearLozenges();
            this.componentsMultiSelect.clearLozenges();
            this.testEnvironmentsLabelPicker.clearLozenges();

            this.selectedTestPath = null;
            this.selectedTestSetKey = null;
            this.lastAssigneeFilterValue = null;
            this.lastStatusFilterValue = null;
            this.lastComponentsValue = null;
            this.lastTestEnvironmentsValue = null;
            this.lastTextFilterValue = null;

            $('#testplan-tests-filter-contains').removeClass('disabled');
            $('#testplan-tests-filter-testsets').removeClass('disabled');
            $('#testplan-tests-filter-assignee').removeClass('disabled');
            $("#testplan-tests-filter-component").removeClass('disabled');
            $("#testplan-tests-filter-test-environment").removeClass('disabled');
            $('#testplan-tests-filter-status').removeClass('disabled');

            this.mainView.cleanFilters();

            $("#testplan-tests-filter-testsets-single-select").removeClass("aui-ss-has-entity-icon");
            $("#testplan-tests-filter-assignee-single-select").removeClass("aui-ss-has-entity-icon");

            return false;
        }
    });


})(AJS.$);
