(function($) {

	RAVEN.namespace("RAVEN.TestPlan");

	RAVEN.TestPlan.InnerTableMetaModel = Backbone.Model.extend({
		testKey : null,
		environments: null,
		
		initialize : function(attributes, options) {
			this.testKey = options.test;
			this.environments = options.environments;
		},
		
		urlRoot : function() {
			return contextPath + '/rest/raven/1.0/testplan/' +this._getTestPlanKey() + '/innertestexecutions/' + this.testKey + '?testEnvironments=' + this._getTestEnvironments();
		},
		
        defaults: function() {
            return [];
        },

        _getTestPlanKey: function () {
            return $('#raven-testplan-tests-issue-key').val();
        },

		_getTestEnvironments: function() {
            if(this.environments != null) {
                return this.environments;
            }
            return '';
		}
	});
	
	RAVEN.TestPlan.InnerTableView = Backbone.View.extend({
		el: "#raven-plan-innerTable",
		
		testPlanKey: null,
		
		testKey: null,
		eEnvironments: null,
		
        /**
         * Permissions
         */
        canExecuteInline: false,
        canEdit: false,
        canAssign: false,
        canExecuteTestStatus: false,
        
        initialize: function(parameters) {
        	var self = this;

            this.model = new RAVEN.TestPlan.InnerTableMetaModel({},parameters);

            // this._delegateUnscopedEvents(); TODO PROBABLY NEED
            
            AJS.messages.setup();
        	
        	this.testPlanKey = $('#raven-testplan-tests-issue-key').val();

            this.canExecuteInline = $('#can-execute-inline').val() === 'true';
            this.canEdit = $('#can-edit').val() === 'true';
            this.canAssign = $('#can-assign').val() === 'true';

            this.testKey = parameters.test;
            this.environments = parameters.environments;

            $('#plan-entries-table tr#' + this.testKey + ' td').block()
            $('#raven-plan-innerTable').block(); // TODO might not need no table yet
            this.model.fetch({
                success: function() {
                    self.render();
                    $('#raven-plan-innerTable').unblock();
                    $('#plan-entries-table tr#' + self.testKey + ' td').unblock();
                },
                error: function(model, response) {
                    self.message(AJS.I18n.getText('view-issue-testplan-tests-section.column-layout.error'), null, 'fatal');
                    $('#raven-plan-innerTable').unblock();
                    $('#plan-entries-table tr#' + self.testKey + ' td').unblock();
                }
            });
        },
        
        render: function() {
        	var values = _.values(this.model.attributes);
        	var canExecuteTest = false;
        	if(values && values.length > 0){
        		canExecuteTest = values[0].canExecuteTest;
        	}
        	var $table = RAVEN.TestPlan.renderInnerTable({
        		testKey: this.testKey,
				environments: this.environments,
        		contextPath: contextPath,
        		testExecutions: values,
        		canEdit: this.canEdit,
        		canAssign: this.canAssign,
        		canExecuteTest: canExecuteTest,
        		canExecuteTestExecution: this.canExecuteTestExecution,
        		canExecuteInline: this.canExecuteInline});
        	$("<tr id='inner-"+this.testKey+"' class='testplanInnerTable'><td colspan=100% class='innertable_wrapper' id="+this.testKey+"></td></tr>").insertAfter($("#"+this.testKey));
        	$(".innertable_wrapper#"+this.testKey).append($table);
        	
        	var rows = $("#inner-" + this.testKey + " .test-exec-entry");
        	for(var i = 0; i < values.length; i++) {
        		if(values[i].testRunAssignee) {
        			$(rows[i]).attr("data-test-run-assignee",JSON.stringify(values[i].testRunAssignee).replace(/'/g, "\\'"));
        		}
        	}
        	
            //add tipsy for non-executable tests
            $(".not-executable-test-testplan").tipsy();
        },
        
        destroy: function () {
        	
        },
        
        _cleanView: function() {
        	
        },
        
	});
	
})(AJS.$);