(function($) {

	RAVEN.namespace("RAVEN.Project.Test");
	
    /**
     * This Model defines the column layout for the given user
     */
    RAVEN.Project.Test.TestTableMetaModel = Backbone.Model.extend({
        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/test/columns';
        },

        defaults: function() {
            return [];
        }
    });
	
	 /**
     * This is the main view for the Test Runs table on a Test Execution issue
     */
    RAVEN.Project.Test.TestsView = Backbone.View.extend({
    	el: '#raven-test',
    	
        /**
         * DataTable Instance
         */
    	testTable: null,
    	
    	userColumns: [],
    	
    	filterVersionPicker: null,
        
    	analisysVersionPicker: null,

        testEnvironmentPicker: null,
    	
    	message: function(title, message, type) {
    		clearMessage();
    		AJS.messages.error('#raven-test-message-bar', {
    			title : title,
    			body : message
    		});
    	},
    	
    	clearMessage: function() {
    		$('#raven-test-message-bar').empty();
    	},

    	refresh: function(){
    		this._cleanView();
    		this.initialize();
    		//this.refreshDataTable();
    	},
    	
    	initialize: function (){
    		var self = this;
    		
    		this.model = new RAVEN.Project.Test.TestTableMetaModel();
    
	        this.model.fetch({
	            success: function() {
	                //$('#raven-test-testruns').unblock();
	                if (self.$el.closest(document.documentElement).length > 0) {
	                    self.render();
	                }
	            },
	            error: function(model, response) {
	                self.message(AJS.I18n.getText('projectpanels-section.column-layout.error'), null, 'fatal');
	            }
	        });
	        
	   	 // render test runs toolbar view
	     //   this._renderTestRunsToolbarView();

    	},
    	
        render: function() {
        	var self = this;
        	$(document).on('submit.raven-test-panel', '#raven-test-panel-form', function(e) {
                self.refreshDataTable();
                return false;
            });
        		
            $(document).on('click.raven-test-panel', '#raven-test-panel-clear-filters', function(e) {
	            if (self.filterVersionPicker) {
	                self.filterVersionPicker._setDescriptorWithValue("-1");
	            }
	            
	            if(self.filterComponentPicker) {
	            	self.filterComponentPicker.clearLozenges();
	            }
	            $('#raven-test-filter-contains-text').val('');
	
	            self.refreshDataTable();
	        });
        	
	        this.initializefilters();
	        this.createDataTable();
	        
            // render test runs toolbar view
            this._renderTestRunsToolbarView();

            this.ready = true;

            return this;
        },
    	
    	createDataTable: function() {
    		var self = this;
    		
    		var projectKey = $('#raven-test-projectKey').val();

    		var path = contextPath + '/rest/raven/1.0/project/' + projectKey + "/test";

    		var $table = RAVEN.Project.renderTestTable({'columns': _.values(this.model.attributes)});
    		
    		var $tableWrapper = $('#tests-section');
    		$tableWrapper.empty();
    		$tableWrapper.html($table);
    		
    		testTable = $('#test-entries-table').xrayDataTable({
                "oLanguage": {
                    "sEmptyTable": AJS.I18n.getText('projectpanels.test.datatable.empty'),
                    "oPaginate": {
                        "sFirst": AJS.I18n.getText('common.label.first'),
                        "sLast": AJS.I18n.getText('common.label.last'),
                        "sNext": AJS.I18n.getText('common.label.next'),
                        "sPrevious": AJS.I18n.getText('common.label.previous')
                    },
                    "sInfoEmpty": AJS.I18n.getText('common.table.label.no-entries-to-show'),
                    "sEmptyTable": AJS.I18n.getText('projectpanels.test.datatable.empty'),
                    "sInfo": AJS.I18n.getText('common.table.label.showing-records'),
                    "sZeroRecords": AJS.I18n.getText('common.table.label.no-records-found'),
                    "sInfoFiltered": AJS.I18n.getText('common.table.label.table-filtered-from'),
                    "sSearch":  AJS.I18n.getText('common.table.label.search'),
                    "sLengthMenu": AJS.I18n.getText('common.table.label.show-menu-entries')
                },
                "bServerSide" : true,
                "fnServerData": function( sSource, aoData, fnCallback, oSettings){
                    self._fnServerData( sSource, aoData, fnCallback, oSettings );
                },
                "sAjaxSource": path,
    			"sAjaxDataProp" : "entries",
    			"bProcessing": false,
    			"bAutoWidth" : false,
    			"bDestroy": true,
                "bDefaultFontSize": true,
    			"dataType": "json",
    			"type":"GET",
    			"sPaginationType" : "full_numbers",
                "bFilter": false,
    			"bSort" : true,
    			"aaSorting" : [],
    			"aoColumnDefs" : this._createTestRunTableColumnDefs(),
    			"aoColumns" : this._createTestRunTableColumns(),
                "fnInitComplete": function(oSettings, json) {
                    
                },
    		});
    	},
    	
    	 /**
         * Creates the DataTable column descriptors dynamically based on the user column layout
         */
    	_createTestRunTableColumns: function() {
            var userColumnDefs = this._createTestRunUserColumns();

            var lastColumnDefs = [{
    			"mDataProp" : "status",
    			"bSortable" : true,
    			"bSearchable" : false,
    			"mRender" : function(data, type, row) {
    				var statusLabel = null;
    				if (data.versionName && data.versionName.length > 0) {
    					statusLabel = data.versionName + ' - ' + data.name;
    				} else {
    					statusLabel = data.name;
    				}

    				return '<span data-status-id="' + data.id + '" class="aui-lozenge" style="background-color: ' + data.color + '; color: white; text-transform: none; min-width: 100px;" title="' + data.description + '">' + statusLabel + '</span>';
    			}
    		},
    		{
    			"bSortable" : false,
    			"bSearchable" : false,
    			"mDataProp" : null,
    			"sClass" : "nav issue_actions",
    			"sDefaultContent" : '',
    			"mRender" : function(data, type, row) {
    				return '<a href="#'
    						+ row["key"]
    						+ '" aria-owns="'
    						+ row["key"]
    						+ '" aria-haspopup="true" class="aui-dropdown2-trigger aui-dropdown2-trigger-arrowless aui-style-default icon-tools-small"></a>'
    						+ '<div id="'
    						+ row["key"]
    						+ '" class="aui-dropdown2 aui-style-default">'
    						+ '<ul class="aui-list-truncate">'
    						+ '<li><a id="'
    						+ row["id"]
    						+ '" href="' + contextPath + '/secure/EditIssue!default.jspa?id='
    						+ row["id"]
    						+ '&amp;returnUrl='
    						+ window.location
    						+ '" class="editTestAction aui-list-item-link">'+AJS.I18n.getText('common.action.edit')+'</a></li>'
    						/*+ '<li><a id="'
    						+ row["key"]
    						+ '" href="javascript:void(0);" class="addTestAction aui-list-item-link">Add Test</a></li>' */
    						+ '<li><a href="' + contextPath + '/secure/DeleteIssue!default.jspa?id='
    						+ row["id"]
    						+ '&amp;returnUrl='
    						+ window.location
    						+ '">'+AJS.I18n.getText('common.action.delete')+'</a></li>'
    						+ '</ul></div>';

    			}
    		}];

            return userColumnDefs.concat(lastColumnDefs);
        },
        
        /**
         * Creates the DataTable column def descriptors dynamically based on the user column layout
         */
        _createTestRunTableColumnDefs: function() {
            var self = this;
            var userColumnDefs = this._createDataTableUserColumnDefs();

            var lastColumnDefs = [ {
    			"mData" : "status",
    			"bSortable" : true,
    			"sWidth" : "110px",
    			"mRender" : function(data, type, row) {
    				if (data) {
    					return "<span data-status-id="
    							+ data.id
    							+ " class=\"aui-lozenge testentry-progress-bar\" style=\"background-color:"
    							+ data.color + "\" title=\"" + data.description
    							+ "\">" + data.name + "</span>";
    				} else {
    					return "-";
    				}
    			},
    			"aTargets" : [_.values(this.model.attributes).length]
    		},
    		{
    			"mData" : "status",
    			"bSortable" : true,
    			"sWidth" : "110px",
    			"mRender" : function(data, type, row) {
    				if (data) {
    					return "<span data-status-id="
    							+ data.id
    							+ " class=\"aui-lozenge testentry-progress-bar\" style=\"background-color:"
    							+ data.color + "\" title=\"" + data.description
    							+ "\">" + data.name + "</span>";
    				} else {
    					return "-";
    				}
    			},
    			"aTargets" : [_.values(this.model.attributes).length + 1]
    		}
    		];
    		
    		
    		return userColumnDefs.concat(lastColumnDefs);
        },
        
        /**
         * Creates the DataTable column def for user columns
         */
        _createDataTableUserColumnDefs: function() {
			
    		return _.map(_.values(this.model.attributes), function(column, index) {
    			console.log('index: ' + index);
				return {
					"mData" : 'userColumns.' + column.id,
					"bSortable" : column.sortable,
					"bSearchable" : false,
					"aTargets" : [ index ]
				};
			});
    	},
    	
    	_createTestRunUserColumns:function() {
		   
		   return _.map(_.values(this.model.attributes), function(column, index) {
			   return {
	               "mData": 'userColumns.' + column.id
	           };
			 });
		   
	   },
	   
	   _fnServerData: function( sSource, aoData, fnCallback, oSettings ) {
	        $('#raven-test').block();

	        // set filter
	        this.populateFilterQueryParams(aoData);

	        oSettings.jqXHR = $.ajax( {
	            "dataType": 'json',
	            "type": "GET",
	            "url": sSource,
	            "data": aoData,
	            "error": function(qXHR, textStatus, errorThrown) {
	                if (errorThrown && errorThrown.trim() != '') {
	                    errorThrown = errorThrown + ': ' + qXHR.responseText;
	                } else {
	                    errorThrown = qXHR.responseText;
	                }
	                message(AJS.I18n.getText('projectpanels.test.datatable.processingerror'), errorThrown)
	            },
	            "complete": function() {
	                $('#raven-test').unblock();
	            },
	            "success": fnCallback
	        });
	    },
	    
	    populateFilterQueryParams: function(aoData) {
	        var versionValue = $('#raven-test-filter-version').val();
	        if (versionValue && versionValue != -1) {
	            aoData.push({ "name": "version", "value": versionValue});
	        }

	        var searchValue = $('#raven-test-filter-contains-text').val();
	        if (searchValue) {
	            aoData.push({ "name": "contains", "value": searchValue});
	        }

            var environmentAnalysisValue = $('#raven-test-analysis-environment').val();
            if (environmentAnalysisValue && environmentAnalysisValue != -1) {
                aoData.push({ "name": "analysisEnvironment", "value": environmentAnalysisValue});
            }

	        var versionAnalysisValue = $('#raven-test-analysis-version').val();
	        if (versionAnalysisValue) {
	            aoData.push({ "name": "analysisVersion", "value": versionAnalysisValue});
	        }
	        
	        var componentValue = $('#raven-test-filter-component').val();
	        if(componentValue) {
	        	aoData.push({ "name": "components", "value": componentValue});
	        }
	    },

	    refreshDataTable: function() {
	        if ($('#test-entries-table').length > 0) {
	            var table = $('#test-entries-table').xrayDataTable();
	            table.fnDraw(false);
	        }
	    },
	    
        _cleanView: function() {
            if (this.dataTable) {
                this.dataTable.fnDestroy(true);
                this.dataTable = null;
            }

            this.model = null;

            var $tableWrapper = this.$el.find('#tests-section');

          //  $('#raven-test-filter-version').remove();
          //  $('#raven-test-analysis-version').remove();
            $tableWrapper.empty();
        },
	    
	    initializefilters: function(){
			var projectKey = $('#raven-test-projectKey').val();
			var self = this;
			
			if(!this.filterVersionPicker){
				this.filterVersionPicker = this.createVersionPicker($('#raven-test-filter-version'));
			}
			
			if(!this.analisysVersionPicker){
				this.analisysVersionPicker = this.createVersionPicker($('#raven-test-analysis-version'));
			}
			
			if(!this.filterComponentPicker) {
				this.filterComponentPicker = this.createComponentPicker($("#raven-test-filter-component"));
			}

            if(!this.testEnvironmentPicker){ 
                this.testEnvironmentPicker = this._createTestEnvironmentPicker($('#raven-test-analysis-environment'));
            }

            $('#raven-test-analysis-environment').on('change', function(e) {
                self.refreshDataTable();
            });
			
	        $('#raven-test-analysis-version').on('change', function(e) {
	        	self.refreshDataTable();
	        });

			$('#raven-test-panel-form').show();
		},
		
		createVersionPicker: function($selectField) {
	        $selectField.on('initialized', function(e, field) {            
	            var version = $('body').data('version');
	            if (!version || version.toString().charAt(0) === '5') {
	                $('input', $selectField.parent()).height('23px');
	            }
	        });
	        
	        return new AJS.SingleSelect({
	            element: $selectField,
	            showDropdownButton: true,
	            revertOnInvalid: true,
	        });
	    },
	    
		createComponentPicker: function($selectField) {
	    	return new AJS.MultiSelect({
				element: $selectField,
				itemAttrDisplayed: "label",
				showDropdownButton: true,
				submitInputVal: false,
				noQueryNoRequest: false,
				revertOnInvalid: true,
				loadOnInit: true,
				errorMessage: "",
				ajaxOptions: {
					query: true,
					url: contextPath + '/rest/api/2/project/' + $('#raven-test-projectKey').val() + '/components',
					formatResponse: function (data) {
						var optgroup = new AJS.GroupDescriptor({
							weight: 0,
							id: "all-filter-opt-group",
							replace: true
						});

						$(data).each(function(i, status) {
							this.label = this.name;
							var item = new AJS.ItemDescriptor({
								value: this.name,
								label: this.name,
								title: this.description,
								
								highlighted: true,
							});
							optgroup.addItem(item);
						});

						return [optgroup];
					}
				}
			});
	    },

        _createTestEnvironmentPicker: function ($selectField) {
            var testEnvironmentsCFID = $("#raven-testenvironments-cf-id").val();

            return new AJS.SingleSelect({
                element: $selectField,
                showDropdownButton: true,
                submitInputVal: true,
                noQueryNoRequest: false,
                revertOnInvalid: true,
                loadOnInit: true,
                ajaxOptions: {
                    query: true,
                    url: contextPath + "/rest/api/1.0/labels/suggest?customFieldId=" + testEnvironmentsCFID,
                    formatResponse: this._formatEnvironmentFilterResponse
                }
            });
        },

        _formatEnvironmentFilterResponse: function(data) {
            var ret = [];

            ret.push(new AJS.ItemDescriptor({
                value: "-1",
                label: AJS.I18n.getText("common.label.all"), // title of lozenge
                icon: null,
                allowDuplicate: false,
                highlighted: true
            }));

            ret.push(new AJS.ItemDescriptor({
                value: "__NULL__",
                label: AJS.I18n.getText("common.label.empty"), // title of lozenge
                icon: null,
                allowDuplicate: false,
                highlighted: true
            }));

            if (data && data.suggestions) {
                var suggestionsGroupDescriptor = new AJS.GroupDescriptor({
                    weight: 0, // order or groups in suggestions dropdown
                    label: AJS.I18n.getText("common.label.available-environment-suggestions")
                });

                $(data.suggestions).each(function(i, env) {
                    var item = new AJS.ItemDescriptor({
                        value: env.label,
                        label: env.label,
                        allowDuplicate: false
                    });
                    suggestionsGroupDescriptor.addItem(item);
                });

                ret.push(suggestionsGroupDescriptor);
            }

            return ret;
        },
	    
	    _renderTestRunsToolbarView: function() {
	        var $wrapper = $('.xrayDataTables_wrapper');
	        // if filter already exists then return
	        if ($wrapper.find('#toolbar-wrapper').length > 0) {
	            return;
	        }

	        var toolbarHtml = RAVEN.Project.renderTestRunsToolBar();
	        $wrapper.find('.xrayDataTables_length').after(toolbarHtml);

            this.columnLayoutView = new RAVEN.Project.TestsColumnLayoutView({
                mainView: this
            });
        }
    });   
	 
    RAVEN.Project.Test.TestsView.REFRESH_VIEW = "refreshTestView";
})(AJS.$);