require(['aui/inline-dialog2']);
(function($) {

    var filterValues = {
        "testPlanKey": null,
        "environment": null,
        "version": null,
        "savedFilter": null
    }

    /*we are sending status ids instead of status name (because of translations), so we'll keep
     * this object to map ids with names*/
    var statusesObject = {};

    var TestPlanReportModel = Backbone.Model.extend({
        key: null,
        summary: null,
        testEnvironements: null,
        startDate: null,
        endDate: null,
        totalTests: null,
        statuses: null,
        manual: null,
        cucumber: null,
        generic: null,
        progress: null,
        elapsedTime: null,
        openDefects: null,
        closedDefects: null
    });

    var Filters = Backbone.Model.extend({
        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/report/testplanreport/filterValues';
        }
    });

    RAVEN.Report.TestPlanReportFiltersView = Backbone.View.extend({
        el: "#testplan-filter-content-place-holder",
        projectId: null,
        testPlanView: null,
        htmlFilters: null,
        render: function() {
            var self = this;
            var filters = new Filters();
            var params = $.param({ projectId: this.projectId });
            filters.fetch({
                data: params,
                success: function(data, textStatus, jqXHR) {

                    if (data && data.attributes) {
                        var savedFilter = null;

                        if (JSON.parse(data.attributes.prefsJson).savedFilter != null) {
                            savedFilter = JSON.parse(data.attributes.prefsJson).savedFilter;
                        }

                        self.htmlFilters = RAVEN.Report.renderFilterTestPlans({
                            'versions': data.attributes.versions,
                            'statuses': data.attributes.statuses,
                            'resolutions': data.attributes.resolutions,
                            'requiresExperimentalDialog2': data.attributes.requiresExperimentalDialog2,
                            'userPreferences': JSON.parse(data.attributes.prefsJson),
                            'assigneeUserPref': JSON.parse(data.attributes.prefsJson).assignee,
                            'savedFilter': savedFilter
                        });

                        statusesObject = data.attributes.statuses;

                        self.$el.empty();
                        self.$el.html(self.htmlFilters);

                        self.testPlanView = new RAVEN.Report.testPlanView();
                        self.testPlanView.render();
                    }
                },
                error: function(model, response) {
                    AJS.messages.error($("#message-row-wrapper"), {
                        title: AJS.I18n.getText('report.message-error'),
                        body: response.statusText,
                        closeable: false
                    });
                }

            });
        },

        initialize: function() {
            this.projectId = $("#selected-project-id").val();
            this.render();
        },

        destroy: function() {
            if (this.testPlanView) {
                this.testPlanView.destroy();
                this.testPlanView = null;
            }

            if (this.htmlFilters) {
                $('#raven-testplan-filter-inline-dialog').remove();
                this.htmlFilters = null;
            }
            this.$el.empty();
            this.remove();
        }
    });

    var TestPlans = Backbone.Collection.extend({
        model: TestPlanReportModel,
        url: function() {
            return contextPath + '/rest/raven/1.0/report/testplanreport/testPlans';
        }
    });

    var testStatusModel = Backbone.Model.extend({
        name: null,
        color: null
    });

    var ColumnsLayout = Backbone.Collection.extend({
        model: testStatusModel,
        url: function() {
            return contextPath + '/rest/raven/1.0/report/testplanreport/getDynamicColumns';
        }
    });



    RAVEN.Report.TestPlanReportView = Backbone.View.extend({
        el: "#report-wrapper",
        filterView: null,
        render: function(requiresExperimentalDialog2) {
            var $htmlReport = RAVEN.Report.renderTestPlanReport({ 'requiresExperimentalDialog2': requiresExperimentalDialog2 });
            this.$el.empty();
            this.$el.html($htmlReport);

            /*var testPlanView = new RAVEN.Report.testPlanView();
            testPlanView.render();*/

            /*var filterView = ...*/
            this.filterView = new RAVEN.Report.TestPlanReportFiltersView();
        },
        destroy: function() {
            if (this.filterView) {
                this.filterView.destroy();
                this.filterView = null;
            }
            this.$el.empty();
            // this.remove();
        }
    });

    RAVEN.Report.testPlanView = Backbone.View.extend({
        el: "#testplans-row-wrapper",

        version: null,
        projectId: null,
        execScope: null,
        testPlan: null,
        environment: null,
        versionPicker: null,
        hideTestRunsValue: null,
        start: 0,
        total: 0,
        pageSize: 20,
        prevValues: null,

        FILTERS_MODE: 1,
        SAVED_FILTER_MODE: 2,

        /**
         * Unscoped Events - these events must be registered elsewhere as they will not be
         * fired by backbone because they are not within the view element. Refer to "_delegateUnscopedEvents".
         */
        unscopedEvents: {
            'submit.raven.report.testplan #raven-testplans-filters-form': 'resetTPFilterValues',
            'click.raven.report.testplan #raven-dialog-clear': 'resetTPFilterValues',
            'submit.raven.report.testplan #raven-requirement-savedfilter-form': 'filtersSubmit',
            'click.raven.report.testplan #raven-testplan-filters-dialog-apply': 'filtersSubmit',
            'change.raven.report.testplan #status-filter': 'dialogApply',
            'click.raven.report.testplan #raven-testplan-filter-fixversion-suggestions': 'preventDefault',
            'click.raven.report.testplan #raven-testplan-filter-status-suggestions': 'preventDefault',
            'click.raven.report.testplan #raven-testplan-filter-resolution-suggestions': 'preventDefault',
            'click.raven.report.testplan #raven-testplan-filter-assignee-suggestions': 'preventDefault',
            'click.raven.report.testplan #raven-testplan-filter-saved-suggestions': 'preventDefault',
            'click.raven.report.testplan #raven-testplan-filter-component-suggestions': 'preventDefault',
            'click.raven.report.testplan #raven-testplan-filter-saved-suggestions': 'preventDefault',
            'submit.raven.report.testplan #raven-testplan-form': 'preventDefault',
            'click.raven.report.testplan .item-delete': 'preventDefault',
            'click.raven.report.testplan .filter-title': 'clickTitleNotSelected',
            'click.raven.report.testplan #raven-testplan-dialog-cancel': 'closeDialog',
            'click.raven.report.testplan #dismiss-how-to-read-msg': 'disableHowToRead',
            'click.raven.report.testplan #enable-how-to-read-msg': 'enableHowToRead',
            'click.raven.report.testplan #load-more': 'loadMore',
            'click.raven.report.testplan #load-all-bt': 'loadAll',
            'change.raven.report.testplan #raven-testplan-filter-assignee' : 'removeAssigneeIcon'
        },
        loadAll: function() {
            this.prevValues = null;
            this.pageSize = -1;
            this.render();
        },

        loadMore: function() {
            //load next 20 items
            this.start = this.start + 20;
            if (this.total > this.pageSize) {
                $("#load-more").addClass("lastRowSpin");
                this.render();
            }
        },
        setPaginationInfo: function(dataModelsLength) {
            var entriesMsg = "";
            if (dataModelsLength == -1) {
                $("#load-more").css("display", "none");
                $(".load-all-entries").css("display", "none");
                entriesMsg = AJS.I18n.getText('common.table.label.no-entries-to-show');
            } else {
                var totalShowing = this.start + dataModelsLength;
                if (totalShowing >= this.total || this.total == 0) {
                    $("#load-more").css("display", "none");
                } else {
                    $("#load-more").css("display", "block");
                }
                if (totalShowing >= this.total) {
                    totalShowing = this.total;
                    $(".load-all-entries").css("display", "none");
                } else {
                    $(".load-all-entries").css("display", "block");
                }
                entriesMsg = AJS.I18n.getText('common.label.showing-records', totalShowing, this.total);
            }
            $(".info-entries").empty();
            $(".info-entries").append(entriesMsg);
        },

        render: function() {
            $('#report-wrapper').block();
            $('.raven-requirement-header').block();
            if ($('.lastRowSpin').length) {
                $(".lastRowSpin").spin("medium");
                $("#load-more").css("color", "#eee");
            } else {
                $('#report-wrapper').spin("large");
            }

            this.clearMessage();

            var self = this;
            var $tableWrapper = self.$el;
            var project = $("#selected-project-id").val();
            var columnsLayout = new ColumnsLayout();
            var testPlans = new TestPlans();
            var params = $.param({
                projectId: project
            });
            columnsLayout.fetch({
                data: params,
                success: function(data1, textStatus, jqXHR) {
                    self.checkShowHowToRead();

                    var params = $.param(self.createFilterParams());
                    params = $.param({
                        projectId: project,
                        pageSize: -1,
                        start: self.start,
                        pageSize: self.pageSize,
                        hideTestRuns: self.hideTestRunsValue
                    }) + "&" + params;

                    testPlans.fetch({
                        data: params,
                        success: function(data2, textStatus, jqXHR) {

                            if (data2.models && data2.models.length > 0) {
                                self.total = data2.models[0].attributes.total;
                                self.pageSize = data2.models[0].attributes.pageSize;

                                if (self.prevValues) {
                                    $.merge(self.prevValues, data2.models);
                                    var $table = RAVEN.Report.renderTestPlanRows({
                                        'issues': data2.models,
                                        'contextPath': contextPath,
                                        'statuses': data1.models[0].attributes.statuses,
                                        'testEnvironments': data1.models[0].attributes.testEnvironments
                                    });

                                    var position = $("tr:last").offset().top - 80;
                                    $("#testplans-row-wrapper tbody").append($table);
                                    $("html, body").animate({ scrollTop: position }, 1000);
                                } else {
                                    var $table = RAVEN.Report.renderTestPlansTable({
                                        'issues': data2.models,
                                        'contextPath': contextPath,
                                        'statuses': data1.models[0].attributes.statuses,
                                        'testEnvironments': data1.models[0].attributes.testEnvironments
                                    });

                                    self.prevValues = data2.models;
                                    $tableWrapper.html($table);
                                }

                                self.setPaginationInfo(data2.models.length);
                            } else {
                                if ($tableWrapper) {
                                    $tableWrapper.empty();

                                    self.setPaginationInfo(-1);
                                    self.stopLoading();
                                }
                                AJS.messages.generic($("#message-row-wrapper"), {
                                    body: AJS.I18n.getText('common.table.label.no-records-found'),
                                    closeable: false
                                });
                            }
                            var exportParams = params;
                            $("#raven-report-comon-to-csv-link").attr("href", contextPath + "/plugins/servlet/raven/report/testplan/export?" + exportParams);

                            self.stopLoading();
                        },
                        error: function(model, response) {
                            if ($tableWrapper) {
                                $tableWrapper.empty();

                                self.setPaginationInfo(-1);
                                self.stopLoading();
                            }
                            AJS.messages.error($("#message-row-wrapper"), {
                                title: AJS.I18n.getText('report.message-error'),
                                body: response.statusText,
                                closeable: false
                            });
                            self.stopLoading();
                        }
                    });
                },
                error: function() {
                    self.setPaginationInfo(-1);
                    AJS.messages.error($("#message-row-wrapper"), {
                        title: AJS.I18n.getText('projectpanels.report.column-layout.test-statuses.error'),
                        body: response.statusText,
                        closeable: false
                    });

                    self.stopLoading();
                }
            });
        },

        stopLoading: function() {
            $('#report-wrapper').unblock();
            $('.raven-requirement-header').unblock();
            $('#report-wrapper').spinStop();
            $(".lastRowSpin").spinStop();
            $(".lastRowSpin").removeClass("lastRowSpin");
            $("#load-more").css("color", "#999");
            $(".lastRowSpin").unblock();
        },

        initialize: function() {
            var self = this;
            $(".load-all-entries").css("display", "block");

            this.projectId = $("#selected-project-id").val();

            this.initSelectPickers();
            // init filter values and render filter summary
            this.initFilterValues();

            this.renderFilterSummary();

            this._delegateUnscopedEvents();

            if (filterValues && filterValues.savedFilter) {
                this.toogleFilters($('.filter-title[data-mode=2]'));
            }
        },

        toogleFilters: function(trigger) {
            $('.filter-title.selected').removeClass('selected');

            $(trigger).addClass('selected');

            if ($(trigger).data('mode') == this.SAVED_FILTER_MODE) {
                $('#raven-testplans-filters-form').hide();
                $('#raven-requirement-savedfilter-form').show();
            } else {
                $('#raven-testplans-filters-form').show();
                $('#raven-requirement-savedfilter-form').hide();
            }
        },
        checkShowHowToRead: function() {
            var self = this;
            $.ajax({
                type: "GET",
                url: contextPath + "/rest/raven/1.0/report/testplanreport/isHowToRead",

                success: function(data, textStatus, jqXHR) {
                    if (data) {
                        self.showHowToRead();
                    } else {
                        self.hideHowToRead();
                    }
                },
                error: function(qXHR, textStatus, errorThrown) {
                    console.log(qXHR);
                }
            });
        },
        setHowToReadServer: function(value) {
            $.ajax({
                type: "POST",
                data: value,
                url: contextPath + "/rest/raven/1.0/report/testplanreport/setHowToRead",
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    console.log(qXHR);
                }
            });
        },

        showHowToRead: function() {
            $(".raven-how-to-read-chart").show("fast");
            $("#raven-notifications-icon").hide("fast");
            $("#enable-how-to-read-msg").hide("fast");
        },
        hideHowToRead: function() {
            $(".raven-how-to-read-chart").hide("fast");
            $("#raven-notifications-icon").show("fast");
            $("#enable-how-to-read-msg").show("fast");
        },

        includePreviousVersions: function() {
            return $('#include-previous-versions-option').is(':checked');
        },
        destroy: function() {

            // COMPLETELY UNBIND THE VIEW
            $("#show-tooltip").remove();
            $('#load-all-warning').remove();
            this.undelegateEvents();

            this._undelegateUnscopedEvents();
            this.remove();
        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        _delegateUnscopedEvents: function() {
            var self = this;
            $(document).off('.raven.report.testplan');

            $.each(this.unscopedEvents, function(key, val) {
                var eventName = key.substring(0, key.indexOf(' '));
                var eventTrigger = key.substring(key.indexOf(' ') + 1);

                $(document).on(eventName, eventTrigger, $.proxy(self[val], self));
            });
        },

        _undelegateUnscopedEvents: function() {
            $(document).off('.raven.report.testplan');
        },
        removeAssigneeIcon: function(){
            if (!assigneeFieldPicker._hasIcon()){
                if(assigneeFieldPicker.$entityIcon !== undefined){
                    assigneeFieldPicker.$entityIcon.remove();
                }
                assigneeFieldPicker.$container.removeClass("aui-ss-has-entity-icon");
            }
        },

        resetTPFilterValues: function() {
            if (fixVersionFieldPicker) {
                fixVersionFieldPicker._setDescriptorWithValue("-1");
            }
            if (statusFieldPicker) {
                statusFieldPicker.clearLozenges();
            }
            if (resolutionFieldPicker) {
                resolutionFieldPicker._setDescriptorWithValue("-1");
            }
            if (assigneeFieldPicker) {
                assigneeFieldPicker.clear();
                assigneeFieldPicker._setDescriptorWithValue("-1");
            }

            $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-contains').val("");

            var fieldPicker = $('#raven-testplan-filter-saved').data('field-support-obj');
            if (fieldPicker) {
                fieldPicker.clear();
            }
        },

        filtersSubmit: function() {
            this.prevValues = null;
            this.start = 0;
            this.pageSize = 20;

            this.saveFilterValues();
            this.render();

            this.renderFilterSummary();

            if ($('#raven-testplan-filter-inline-dialog').attr('open') == "open") {
                $('#raven-testplan-filter-inline-dialog').removeAttr('open');
            } else {
                document.querySelector('#raven-testplan-filter-inline-dialog').hide();
            }

            return false;
        },

        dialogApply: function() {
            this.saveFilterValues();
            this.render();
            this.renderFilterSummary();

            if ($('#raven-testplan-filter-inline-dialog').attr('open') == "open") {
                $('#raven-testplan-filter-inline-dialog').removeAttr('open');
            } else {
                document.querySelector('#raven-testplan-filter-inline-dialog').hide();
            }
        },

        preventDefault: function(e) {
            console.log("preventDefault");
            e.stopPropagation();
            e.preventDefault();
        },

        clickTitleNotSelected: function(e) {
            this.toogleFilters(e.currentTarget);
            e.stopPropagation();
        },

        closeDialog: function() {
            // Cancel button inline Dialog
            if ($('#raven-testplan-filter-inline-dialog').attr('open') == "open") {
                $('#raven-testplan-filter-inline-dialog').removeAttr('open');
            } else {
                document.querySelector('#raven-testplan-filter-inline-dialog').hide();
            }
        },

        disableHowToRead: function() {
            $('.header-filter').attr('data-initialized', false);
            this.hideHowToRead();
            this.setHowToReadServer("disabled");
        },

        enableHowToRead: function() {
            $('.header-filter').attr('data-initialized', false);
            this.showHowToRead();
            this.setHowToReadServer("enabled");
        },

        /****************************************************************
         *                      END EVENT HANDLERS
         ****************************************************************/
        createFilterParams: function() {
            var filters = {};

            if ($('.filter-title.selected').data('mode') == this.SAVED_FILTER_MODE) {
                var savedFilter = filterValues.savedFilter;
                if (savedFilter) {
                    filters["savedFilter"] = savedFilter;
                }
            } else {
                var fixVersion = filterValues.fixVersion;
                if (fixVersion && fixVersion != -1) {
                    filters["fixVersion"] = fixVersion;
                    filters["previousVersions"] = filterValues.includePreviousVersions;
                }
                var status = filterValues.statusId;
                if (status) {
                    var len = status.length;
                    filters["statusId"] = "";
                    $.each(status, function(index) {
                        filters["statusId"] = filters["statusId"] + this;
                        if (index < len - 1) {
                            filters["statusId"] = filters["statusId"] + ",";
                        }
                    });
                }
                var resolution = filterValues.resolution;
                if (resolution && resolution != -1) {
                    filters["resolution"] = resolution;
                }
                var assignee = filterValues.assignee;
                if (assignee && assignee != -1) {
                    filters["assignee"] = assignee;
                }
                var contains = filterValues.contains;
                if (contains) {
                    filters["contains"] = contains;
                }

                var components = filterValues.components;
                if (components) {
                    filters["components"] = components;
                }
            }

            return filters;
        },

        initSelectPickers: function() {

            // Filter pickers
            fixVersionFieldPicker = this.createSingleSelectPicker($('#raven-testplan-filter-inline-dialog #raven-testplan-filter-fixversion'));
            statusFieldPicker = this.createStatusPicker($('#raven-testplan-filter-inline-dialog #raven-testplan-filter-status'));
            resolutionFieldPicker = this.createSingleSelectPicker($('#raven-testplan-filter-inline-dialog #raven-testplan-filter-resolution'));
            assigneeFieldPicker = this.initAssigneeFilterPicker($('#raven-testplan-filter-assignee'));
            this.createSingleFilterPickers();
        },
        initAssigneeFilterPicker: function(e) {
            var $assineePicker = e;

            if ($assineePicker.data("aui-ss")) return;

            var restPath = "/rest/api/1.0/users/picker";

            var data = {
                    showAvatar: true
                },
                inputText = $assineePicker.data('inputValue');

            return new AJS.SingleSelect({
                element: $assineePicker,
                submitInputVal: true,
                revertOnInvalid: true,
                showDropdownButton: !!$assineePicker.data('show-dropdown-button'),
                errorMessage: AJS.I18n.getText("user.picker.invalid.user", "'{0}'"),
                ajaxOptions: {
                    url: contextPath + restPath,
                    query: true, // keep going back to the sever for each keystroke
                    data: data,
                    formatResponse: function(data) {
                        var res = [];
                        var defaultGroupDescriptor = new AJS.GroupDescriptor({
                            weight: 0 // order of groups in suggestions dropdown
                        });
                        defaultGroupDescriptor.addItem(new AJS.ItemDescriptor({
                            value: "-1",
                            label: AJS.I18n.getText("raven-admin-config.general.label.all")
                        }));
                        res.push(defaultGroupDescriptor);

                        jQuery(data).each(function(i, suggestions) {
                            var groupDescriptor = new AJS.GroupDescriptor({
                                weight: 1, // order or groups in suggestions dropdown
                                label: suggestions.footer
                            });

                            jQuery(suggestions.users).each(function(){
                                groupDescriptor.addItem(new AJS.ItemDescriptor({
                                    value: this.name, // value of item added to select
                                    label: this.displayName, // title of lozenge
                                    html: this.html,
                                    icon: this.avatarUrl,
                                    allowDuplicate: false,
                                    highlighted: true
                                }));
                            });
                            res.push(groupDescriptor);
                        });
                        return res;
                    }
                },
                inputText: inputText
            });
        },

        initFilterValues: function() {
            filterValues.savedFilter = $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-saved').val();
            filterValues.fixVersion = $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-fixversion').val();
            filterValues.includePreviousVersions = this.includePreviousVersions();
            var statusesIds = [];
            var arr = $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-status').val()
            if (arr != null) {
                for (var i = 0; i < arr.length; i++) {
                    for (var j = 0; j < statusesObject.length; j++) {
                        if (arr[i] == statusesObject[j].name) {
                            statusesIds.push(statusesObject[j].id);
                        }
                    }
                }
            }
            filterValues.statusId = (statusesIds.length > 0) ? statusesIds : null;
            filterValues.resolution = $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-resolution').val();
            filterValues.assignee = $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-assignee').val();
            filterValues.contains = $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-contains').val();
            filterValues.components = $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-component').val();
        },
        saveFilterValues: function() {
            if ($('.filter-title.selected').data('mode') == this.SAVED_FILTER_MODE) {
                filterValues.savedFilter = $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-saved').val();
                filterValues.fixVersion = null;
                filterValues.includePreviousVersions = null;
                filterValues.statusId = null;
                filterValues.resolution = null;
                filterValues.assignee = null;
                filterValues.contains = null;
                filterValues.components = null;
            } else {
                filterValues.savedFilter = null;
                filterValues.fixVersion = $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-fixversion').val();
                filterValues.includePreviousVersions = this.includePreviousVersions();
                var statusesIds = [];
                var arr = $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-status').val();
                if (arr != null) {
                    for (var i = 0; i < arr.length; i++) {
                        for (var j = 0; j < statusesObject.length; j++) {
                            if (arr[i] == statusesObject[j].name) {

                                statusesIds.push(statusesObject[j].id);

                            }
                        }
                    }
                }
                filterValues.statusId = (statusesIds.length > 0) ? statusesIds : null;
                filterValues.resolution = $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-resolution').val();
                filterValues.assignee = $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-assignee').val();
                filterValues.contains = $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-contains').val();
                filterValues.components = $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-component').val();
            }
        },

        renderFilterSummary: function() {
            var summary = '';
            if (filterValues.savedFilter && filterValues.savedFilter != "-1") {
                summary = '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.saved-filter') + '</b>  ' + $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-saved option[value="' + filterValues.savedFilter + '"]').text();
            } else {
                if (filterValues.fixVersion && filterValues.fixVersion != "-1") {
                    var previousVersions = filterValues.includePreviousVersions;
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.fix-version') + '</b>  ';
                    if (previousVersions === true) {
                        summary += '<= ';
                    }
                    summary += $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-fixversion option[value="' + filterValues.fixVersion + '"]').text();
                }
                if (filterValues.statusId) {
                    if (summary) {
                        summary += '; ';
                    }
                    var statusesNames = "";
                    for (var i = 0; i < filterValues.statusId.length; i++) {
                        for (var j = 0; j < statusesObject.length; j++) {
                            if (filterValues.statusId[i] == statusesObject[j].id) {
                                if (i > 0 && i < filterValues.statusId.length) {
                                    statusesNames = statusesNames + ",";
                                }
                                statusesNames = statusesNames + statusesObject[j].name;
                            }
                        }
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.status') + '</b>  ' + statusesNames;
                }
                if (filterValues.resolution && filterValues.resolution != "-1") {
                    if (summary) {
                        summary += '; ';
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.resolution') + '</b>  ' + $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-resolution option[value="' + filterValues.resolution + '"]').text();
                }
                if (filterValues.assignee && filterValues.assignee != "-1") {
                    if (summary) {
                        summary += '; ';
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.assignee') + '</b>  ' + $('#raven-testplan-filter-inline-dialog #raven-testplan-filter-assignee option[value="' + filterValues.assignee + '"]').text();
                }
                if (filterValues.contains) {
                    if (summary) {
                        summary += '; ';
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.contains') + '</b>  ' + filterValues.contains;
                }
                if (filterValues.components) {
                    if (summary) {
                        summary += '; ';
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.component') + '</b>  ' + filterValues.components;
                }
            }

            if (summary) {
                $('#raven-filter-trigger').html(summary);
            } else {
                $('#raven-filter-trigger').text(AJS.I18n.getText('projectpanels.requirement-coverage.label.filter-button'));
            }
        },
        createStatusPicker: function($selectField) {
            return new AJS.MultiSelect({
                element: $selectField,
                showDropdownButton: true,
                revertOnInvalid: true,
            });
        },
        createSingleSelectPicker: function($selectField) {
            $selectField.on('initialized', function(e, field) {
                var version = $('body').data('version');
                if (!version || version.toString().charAt(0) === '5') {
                    $('input', $selectField.parent()).height('23px');
                }
            });

            return new AJS.SingleSelect({
                element: $selectField,
                showDropdownButton: true,
                revertOnInvalid: true,
            });
        },

        createSingleFilterPickers: function(ctx) {

            var restPath = "/rest/gadget/1.0/pickers/filters";

            $(".raven-default-filter-picker", ctx).each(function() {
                var $this = $(this);
                if ($this.data("aui-ss")) return;
                var data = { showAvatar: true },
                    inputText = $this.data('inputValue');

                var field = new AJS.SingleSelect({
                    element: $this,
                    submitInputVal: true,
                    //revertOnInvalid: true,
                    showDropdownButton: !!$this.data('show-dropdown-button'),
                    errorMessage: AJS.format("filters.picker.invalid.filter", "'{0}'"),
                    ajaxOptions: {
                        url: contextPath + restPath,
                        query: true, // keep going back to the sever for each keystroke
                        data: data,
                        formatResponse: function(data) {
                            var savedFilters = [];
                            if (data.filters) {
                                $.each($(data.filters), function(index, filter) {
                                    savedFilters[index] = new AJS.ItemDescriptor({
                                        value: filter.id,
                                        label: filter.name
                                    });
                                });
                            }
                            return savedFilters;
                        }
                    },
                    inputText: inputText
                });

                $this.data('field-support-obj', field);
            });
        },

        clearMessage: function() {
            $('#message-row-wrapper').empty();
        }
    });

})(AJS.$);